@@ -139,6 +139,11 @@ public class CDFConvert extends JFrame
     JRadioButton encoding13 = new JRadioButton("AlphaVMSg");
     JRadioButton encoding14 = new JRadioButton("AlphaVMSi");
     JRadioButton encoding15 = new JRadioButton("AlphaOSF1");
+    JRadioButton encoding16 = new JRadioButton("ARM_LITTLE");
+    JRadioButton encoding17 = new JRadioButton("ARM_LITTLE");
+    JRadioButton encoding18 = new JRadioButton("IA64VMSi");
+    JRadioButton encoding19 = new JRadioButton("IA64VMSd");
+    JRadioButton encoding20 = new JRadioButton("IA64VMSg");
 
 /** A group radio buttons for Epoch data type conversion. */
     ButtonGroup epoch = new ButtonGroup();
@@ -547,6 +552,11 @@ public class CDFConvert extends JFrame
         encoding13.setActionCommand("alphavmsg");
         encoding14.setActionCommand("alphavmsi");
         encoding15.setActionCommand("alphaosf1");
+        encoding16.setActionCommand("arm_little");
+        encoding17.setActionCommand("arm_big");
+        encoding18.setActionCommand("ia64vmsi");
+        encoding19.setActionCommand("ia64vmsd");
+        encoding20.setActionCommand("ia64vmsg");
         encoding0.addActionListener(encodingListener);
         encoding1.addActionListener(encodingListener);
         encoding2.addActionListener(encodingListener);
@@ -563,6 +573,12 @@ public class CDFConvert extends JFrame
         encoding13.addActionListener(encodingListener);
         encoding14.addActionListener(encodingListener);
         encoding15.addActionListener(encodingListener);
+        encoding16.addActionListener(encodingListener);
+        encoding17.addActionListener(encodingListener);
+        encoding18.addActionListener(encodingListener);
+        encoding19.addActionListener(encodingListener);
+        encoding20.addActionListener(encodingListener);
+
         encoding0.setSelected(true);
         encoding.add(encoding0);
         encoding.add(encoding1);
@@ -580,6 +596,12 @@ public class CDFConvert extends JFrame
         encoding.add(encoding13);
         encoding.add(encoding14);
         encoding.add(encoding15);
+        encoding.add(encoding16);
+        encoding.add(encoding17);
+        encoding.add(encoding18);
+        encoding.add(encoding19);
+        encoding.add(encoding20);
+
         encoding0.setToolTipText("Use the encoding from the source CDF for the destination CDF");
         encoding1.setToolTipText("Use the host machine encoding for the destination CDF");
         encoding2.setToolTipText("Use the network encoding for the destination CDF");
@@ -596,6 +618,11 @@ public class CDFConvert extends JFrame
         encoding13.setToolTipText("Use the AlphaVMSg encoding for the destination CDF");
         encoding14.setToolTipText("Use the AlphaVMSi encoding for the destination CDF");
         encoding15.setToolTipText("Use the AlphaOSF1 encoding for the destination CDF");
+        encoding16.setToolTipText("Use the ARM_LITTLE encoding from he destination CDF");
+        encoding17.setToolTipText("Use the ARM_BIG encoding for the destination CDF");
+        encoding18.setToolTipText("Use the IA64VMSi encoding for the destination CDF");
+        encoding19.setToolTipText("Use the IA64VMSd encoding for the destination CDF");
+        encoding20.setToolTipText("Use the IA64OSFg encoding for the destination CDF");
 
         encodingPanel.setLayout(new GridLayout(3,6));
         encodingPanel.add(ENCODING);
@@ -615,6 +642,11 @@ public class CDFConvert extends JFrame
         encodingPanel.add(encoding13);
         encodingPanel.add(encoding14);
         encodingPanel.add(encoding15);
+        encodingPanel.add(encoding16);
+        encodingPanel.add(encoding17);
+        encodingPanel.add(encoding18);
+        encodingPanel.add(encoding19);
+        encodingPanel.add(encoding20);
 
         // Initialize the epoch radio button.
         EpochListener epochListener = new EpochListener();
