@@ -1,4 +1,4 @@
-// $Id: CDFDialog.java,v 1.1.1.1 2017/01/04 20:58:10 liu Exp $
+// $Id: CDFDialog.java,v 1.1.1.1 2018/08/29 19:16:17 mhliu Exp $
 /******************************************************************************
 * Copyright 1996-2014 United States Government as represented by the
 * Administrator of the National Aeronautics and Space Administration.
@@ -100,9 +100,14 @@ public class CDFDialog extends JDialog implements ActionListener,
         encodingV.addElement("HP");
         encodingV.addElement("NeXT");
 	encodingV.addElement("ALPHAOSF1");
-        encodingV.addElement("ALPHAVMSg");
         encodingV.addElement("ALPHAVMSd");
+        encodingV.addElement("ALPHAVMSg");
         encodingV.addElement("ALPHAVMSi");
+        encodingV.addElement("ARM_LITTLE");
+        encodingV.addElement("ARM_BIG");
+        encodingV.addElement("IA64VMSi");
+        encodingV.addElement("IA64VMSg");
+        encodingV.addElement("IA64VMSd");
 
         cben = new JLabeledCB("Encoding", encodingV, false, true);
 	cben.addItemListener(new EncodingListener());
