@@ -28,6 +28,7 @@ public class CDFMerge extends JFrame
     JFrame parentDriver;
 /** The dialog for user interface. */
     JDialog dialog;
+    JPanel panelx = new JPanel(new BorderLayout(), true);
 
 /** The FileChooser for the file selection. */
     JFileChooser fileChooser = new JFileChooser();
@@ -370,7 +371,8 @@ public class CDFMerge extends JFrame
         MSTF.setFont(boldFont);
         masterfile.setFont(boldFont);
 
-	dialog.getContentPane().add(panel1,BorderLayout.NORTH);
+//	dialog.getContentPane().add(panel1,BorderLayout.NORTH);
+	panelx.add(panel1,BorderLayout.NORTH);
 	
         // Panel 2
 
@@ -415,7 +417,8 @@ public class CDFMerge extends JFrame
         prefixes.setToolTipText("Enter the prefixeses to be added to the beginning of attribute and variable names");
         prefixes.setFont(boldFont);
 
-        dialog.getContentPane().add(panel2,BorderLayout.CENTER);
+//        dialog.getContentPane().add(panel2,BorderLayout.CENTER);
+        panelx.add(panel2,BorderLayout.CENTER);
 
         // Panel 3 (buttons and text area)
 
@@ -446,9 +449,14 @@ public class CDFMerge extends JFrame
                 }
         };
         scroller.getViewport().add(info);
-        panel3.add(scroller, BorderLayout.SOUTH);
-	dialog.getContentPane().add(panel3, BorderLayout.SOUTH);
+//        panel3.add(scroller, BorderLayout.SOUTH);
+//	dialog.getContentPane().add(panel3, BorderLayout.SOUTH);
+	panelx.add(panel3, BorderLayout.SOUTH);
   	info.setVisible(true);
+
+        dialog.getContentPane().add(panelx, BorderLayout.NORTH);
+        dialog.getContentPane().add(scroller, BorderLayout.CENTER);
+
     }
 
 /** Implement the run method for Runabout interface.
