@@ -1,3 +1,9 @@
+/******************************************************************************
+* Copyright 1996-2014 United States Government as represented by the
+* Administrator of the National Aeronautics and Space Administration.
+* All Rights Reserved.
+******************************************************************************/
+
 import java.awt.*;
 import java.awt.event.*;
 import java.text.*;
@@ -438,8 +444,10 @@ public class EditgEntryTablePanel extends JPanel implements CDFConstants {
 		jj++;
 	     	dataType = en.getDataType();
 	    	numElem = en.getNumElements();
-	      	value = en.getData();
-	      	dataObject[jj][0] = "" + (en.getID() + 1);
+                try {
+	      	  value = en.getData();
+	      	} catch (CDFException ex) {} 
+                dataObject[jj][0] = "" + (en.getID() + 1);
 	      	dataObject[jj][1] = CDFUtils.getStringDataType(dataType) + "/" + 
 					numElem;
 		if (dataType == CDF_EPOCH) {
