@@ -24,6 +24,7 @@ public class SkeletonCDF extends JFrame
     JFrame parentDriver;
 /** The dialog for user interface. */
     JDialog dialog;
+    JPanel panelx = new JPanel(new BorderLayout(), true);
 
 /** The FileChooser for the file selection. */
     JFileChooser fileChooser = new JFileChooser();
@@ -326,7 +327,8 @@ public class SkeletonCDF extends JFrame
 	cdf.setFont(boldFont);
 	SKLT.setFont(boldFont);
   	sklt.setFont(boldFont);
-	dialog.getContentPane().add(panel1,BorderLayout.NORTH);
+//	dialog.getContentPane().add(panel1,BorderLayout.NORTH);
+	panelx.add(panel1,BorderLayout.NORTH);
 	
         // Panel 2
 
@@ -412,7 +414,8 @@ public class SkeletonCDF extends JFrame
         cachesizes.addFocusListener(this);
         cachesizes.setToolTipText("Enter the cache sizes for dotCDF, staging and compression scratch files in 512 bytes buffer");
 
-        dialog.getContentPane().add(panel2,BorderLayout.CENTER);
+//        dialog.getContentPane().add(panel2,BorderLayout.CENTER);
+        panelx.add(panel2,BorderLayout.CENTER);
 
         // Panel 3 (buttons and text area)
 
@@ -443,10 +446,14 @@ public class SkeletonCDF extends JFrame
 		}
 	};
 	scroller.getViewport().add(info);
-	panel3.add(scroller, BorderLayout.SOUTH);
-	dialog.getContentPane().add(panel3, BorderLayout.SOUTH);
+//	panel3.add(scroller, BorderLayout.SOUTH);
+//	dialog.getContentPane().add(panel3, BorderLayout.SOUTH);
+	panelx.add(panel3, BorderLayout.SOUTH);
   	info.setVisible(true);
 
+        dialog.getContentPane().add(panelx, BorderLayout.NORTH);
+        dialog.getContentPane().add(scroller, BorderLayout.CENTER);
+
     }
 
 /** Implement the run method for Runabout interface.
