@@ -43,6 +43,7 @@ public class SkeletonTable extends JFrame
     JPanel panel31 = new JPanel(new GridLayout(1, 3, 50, 20), true);
     JPanel tpanel1 = new JPanel(new BorderLayout(), true);
     JPanel tpanel2 = new JPanel(new BorderLayout(), true);
+    JPanel tpanel3 = new JPanel(new BorderLayout(), true);
 
 /** "Page Output" checkbox. */
     JCheckBox pgc = new JCheckBox("Page Output", false);
@@ -82,15 +83,26 @@ public class SkeletonTable extends JFrame
 /** A radio button panel for zMode. */
     JPanel zmodePanel = new JPanel(true);
 
+/** A group radio buttons for Style. */
+    ButtonGroup style = new ButtonGroup();
+/** A group radio buttons for style. */
+    JRadioButton style0 = new JRadioButton("Style/0");
+    JRadioButton style1 = new JRadioButton("Style/1");
+    JRadioButton style2 = new JRadioButton("Style/2");
+    JRadioButton style3 = new JRadioButton("Style/3");
+    JRadioButton style4 = new JRadioButton("Style/4");
+/** A radio button panel for style. */
+    JPanel stylePanel = new JPanel(true);
+
 /** A group radio buttons for Values. */
     ButtonGroup values = new ButtonGroup();
-/** A group radio buttons for zMode. */
+/** A group radio buttons for Values. */
     JRadioButton values0 = new JRadioButton("None");
     JRadioButton values1 = new JRadioButton("NRV");
     JRadioButton values2 = new JRadioButton("RV");
     JRadioButton values3 = new JRadioButton("All");
     JRadioButton values4 = new JRadioButton("Selected");
-/** A radio button panel for zMode. */
+/** A radio button panel for Values. */
     JPanel valuesPanel = new JPanel(true);
 
 /** "Enter" button. */
@@ -126,6 +138,8 @@ public class SkeletonTable extends JFrame
     JLabel VARB = new JLabel("Variables: ");
 /** A label of "zMode:" for the radio button group zmode. */
     JLabel ZMODE = new JLabel("zMode: ");
+/** A label of "Style:" for the radio button group zmode. */
+    JLabel STYLE = new JLabel("Style: ");
 /** A label of "Values:" for the radio button group values. */
     JLabel VALUES = new JLabel("Values: ");
 
@@ -165,7 +179,8 @@ public class SkeletonTable extends JFrame
     String strlp = " ";
     String strzz = " ";
     String strzm = "";
-    String report, Zmode, caches, temp, vars;
+    String strsty = "";
+    String report, Zmode, caches, temp, vars, estyle;
     String cdfspec = " ";
     String skltspec = "";
     String SKLTfile;
@@ -381,6 +396,38 @@ public class SkeletonTable extends JFrame
         zmodePanel.add(zmode1);
         zmodePanel.add(zmode2);
 	
+        // Initialize the Style choice.
+
+        StyleListener StyleListener = new StyleListener();
+        style0.setActionCommand("0");
+        style1.setActionCommand("1");
+        style2.setActionCommand("2");
+        style3.setActionCommand("3");
+        style4.setActionCommand("6");
+        style0.addActionListener(StyleListener);
+        style1.addActionListener(StyleListener);
+        style2.addActionListener(StyleListener);
+        style3.addActionListener(StyleListener);
+        style4.addActionListener(StyleListener);
+        style4.setSelected(true);
+        style.add(style0);
+        style.add(style1);
+        style.add(style2);
+        style.add(style3);
+        style.add(style4);
+        style0.setToolTipText("Use dd-mom-yyyy hh:mm-ss.mmm for CDF_EPOCH and dd-mom-yyyy hh:mm-ss.mmm.uuu.nnn.ppp for CDF_EPOCH16");
+        style1.setToolTipText("Use yyyymmdd.ttttttt for CDF_EPOCH and yyyymmdd.ttttttttttttttt for CDF_EPOCH16");
+        style2.setToolTipText("Use yyyymmddhhmnss for both CDF_EPOCH and CDF_EPOCH16");
+        style3.setToolTipText("Use yyyy-mm-ddThh:mn:ss.mmmZ for CDF_EPOCH and yyyy-mm-ddThh:mn:ss.mmm.uuu.nnn.pppZ for CDF_EPOCH16");
+        style4.setToolTipText("Use yyyy-mm-ddThh:mn:ss.mmm for CDF_EPOCH or yyyy-mm-ddThh:mn:ss.mmmuuunnnppp CDF_EPOCH16");
+
+        stylePanel.setLayout(new FlowLayout());
+        stylePanel.add(style0);
+        stylePanel.add(style1);
+        stylePanel.add(style2);
+        stylePanel.add(style3);
+        stylePanel.add(style4);
+	
 	// Initialize the values choice.
 
         ValuesListener valuesListener = new ValuesListener();
@@ -433,10 +480,15 @@ public class SkeletonTable extends JFrame
         tpanel1.add(ZMODE,BorderLayout.WEST); 
 	tpanel1.add(zmodePanel, BorderLayout.CENTER);
 	makeJButton(panel2n, tpanel1, 4, 3, 1, 1, 1.0, 1.0);
-        tpanel2.add(VALUES, BorderLayout.WEST); 
-        tpanel2.add(valuesPanel, BorderLayout.CENTER);
+        tpanel2.add(STYLE,BorderLayout.WEST);
+        tpanel2.add(stylePanel, BorderLayout.CENTER);
+        tpanel3.add(VALUES, BorderLayout.WEST); 
+        tpanel3.add(valuesPanel, BorderLayout.CENTER);
 //        makeJButton(panel2c, Box.createRigidArea(hpad10), 1, 0, 1, 1, 1.0, 1.0);
-	makeJButton(panel2c, tpanel2, 1, 0, 1, 1, 1.0, 1.0);
+	makeJButton(panel2c, tpanel2, 0, 0, 1, 1, 1.0, 1.0);
+        makeJButton(panel2c, Box.createRigidArea(vpad10), 0, 1, 1, 1, 1.0, 1.0);
+
+	makeJButton(panel2c, tpanel3, 1, 0, 1, 1, 1.0, 1.0);
 	makeJButton(panel2c, Box.createRigidArea(vpad10), 0, 1, 1, 1, 1.0, 1.0);
         makeJButton(panel2c, Box.createRigidArea(vpad10), 0, 1, 1, 1, 1.0, 1.0);
         makeJButton(panel2s, CASS, 0, 0, 1, 1, 1.0, 1.0);
@@ -450,6 +502,7 @@ public class SkeletonTable extends JFrame
         panel2.add(panel2c, BorderLayout.CENTER);
 	panel2.add(panel2s, BorderLayout.SOUTH);
 	ZMODE.setFont(boldFont);
+	STYLE.setFont(boldFont);
 	VALUES.setFont(boldFont);
 	CASS.setFont(boldFont);
 	cachesizes.setFont(boldFont);
@@ -628,6 +681,9 @@ public class SkeletonTable extends JFrame
                         if (strzm.equals("")) Zmode = " ";
                         else Zmode = "-zmode " + strzm + " ";
 
+                        if (strsty.equals("")) estyle = " ";
+                        else estyle = "-style " + strsty + " ";
+
 			if (skltspec.equals("") || skltspec.trim().equals("")) 
 			  SKLTfile = " ";
 			else {
@@ -652,13 +708,13 @@ public class SkeletonTable extends JFrame
                         else cdfDelimiter = "\"";
 
                         commandX = command  + strzz + struf + stros + strlp +
-				   strds + strpo + report + Zmode + Values +
-				   Caches + SKLTfile + cdfDelimiter + cdfspec +
-				   cdfDelimiter;
+				   strds + strpo + report + estyle + Zmode +
+                                   Values + Caches + SKLTfile + cdfDelimiter +
+                                   cdfspec + cdfDelimiter;
                         /**************
 			commandX = command  + strzz + struf + stros + strlp +
-				   strds + strpo + report + Zmode + Values +
-				   Caches + SKLTfile + " " + cdfspec + " ";
+				   strds + strpo + report + estyle + Zmode +
+                                   Values + Caches + SKLTfile + " " + cdfspec + " ";
                         *************/
                         String OS = System.getProperty("os.name").toLowerCase();
                         if (OS.indexOf("windows") != -1) {
@@ -738,7 +794,16 @@ public class SkeletonTable extends JFrame
         }
     }
 
-/** Action for radio buttons for zMode
+/** Action for radio buttons for Style
+ */
+
+    class StyleListener implements ActionListener {
+        public void actionPerformed(ActionEvent e) {
+           strsty = e.getActionCommand();
+        }
+    }
+
+/** Action for radio buttons for Values
  */
 
     class ValuesListener implements ActionListener {
