@@ -53,6 +53,7 @@ public interface CDFConstants {
     public static final long TT2000_1_STRING_LEN  =  19;
     public static final long TT2000_2_STRING_LEN  =  14;
     public static final long TT2000_3_STRING_LEN  =  29;
+    public static final long TT2000_4_STRING_LEN  =  30;
 
 
     /*----------------*/
@@ -96,6 +97,11 @@ public interface CDFConstants {
     public static final long ALPHAVMSd_ENCODING    =  14;
     public static final long ALPHAVMSg_ENCODING    =  15;
     public static final long ALPHAVMSi_ENCODING    =  16;
+    public static final long ARM_LITTLE_ENCODING   =  17;
+    public static final long ARM_BIG_ENCODING      =  18;
+    public static final long IA64VMSi_ENCODING     =  19;
+    public static final long IA64VMSd_ENCODING     =  20;
+    public static final long IA64VMSg_ENCODING     =  21;
 
     /*----------*/
     /* Decoding */
@@ -118,6 +124,11 @@ public interface CDFConstants {
     public static final long ALPHAVMSd_DECODING      = ALPHAVMSd_ENCODING;
     public static final long ALPHAVMSg_DECODING      = ALPHAVMSg_ENCODING;
     public static final long ALPHAVMSi_DECODING      = ALPHAVMSi_ENCODING;
+    public static final long ARM_LITTLE_DECODING     = ARM_LITTLE_ENCODING;
+    public static final long ARM_BIG_DECODING        = ARM_BIG_ENCODING;
+    public static final long IA64VMSd_DECODING       = IA64VMSd_ENCODING;
+    public static final long IA64VMSg_DECODING       = IA64VMSg_ENCODING;
+    public static final long IA64VMSi_DECODING       = IA64VMSi_ENCODING;
 
     /*------------------------------------*/
     /* Record or dimension variance flags */
@@ -241,6 +252,7 @@ public interface CDFConstants {
 
     public static final double ILLEGAL_EPOCH_VALUE      = -1.0;
     public static final long ILLEGAL_TT2000_VALUE       = -9223372036854775805L;
+                                          /* as 1707-09-22T12:12:10.961224195 */
     public static final long FILLED_TT2000_VALUE         = Long.MIN_VALUE;
 
     /*------------------------------------*/
@@ -401,6 +413,8 @@ public interface CDFConstants {
     public static final long TT2000_USED_OUTDATED_TABLE   = -2234;
     public static final long BADDATE_LEAPSECOND_UPDATED   = -2235;
     public static final long FUNCTION_NOT_SUPPORTED       = -2236;
+    public static final long TRY_TO_READ_NONSTRING_DATA   = -2237;
+    public static final long BAD_NUM_STRINGS              = -2238;
 
 
     /*----------------------------------------------------------------*/
@@ -591,8 +605,17 @@ public interface CDFConstants {
     public static final long rVAR_RECORDS_RENUMBER_ =   157;
     public static final long zVAR_RECORDS_RENUMBER_ =   158;
     public static final long CDF_LEAPSECONDLASTUPDATED_ = 159;
+    public static final long rENTRY_STRINGSDATA_ =      160;
+    public static final long zENTRY_STRINGSDATA_ =      161;
+    public static final long rENTRY_NUMSTRINGS_ =       162;
+    public static final long zENTRY_NUMSTRINGS_ =       163;
 
     public static final long CDFwithSTATS_ = 200;   /* For CDF internal use only! */
     public static final long CDF_ACCESS_ =   201;   /* For CDF internal use only! */
+    public static final String STRINGDELIMITER = new String("\\N ");
+
+    public static final double BeginUnixTimeEPOCH = 62167219200000.0;
+    public static final double BeginUnixTimeEPOCH16 = 62167219200.0;
+
   
 }
