@@ -1,4 +1,4 @@
-//$Id: CDFData.java,v 1.1.1.1 2017/01/04 20:58:09 liu Exp $
+//$Id: CDFData.java,v 1.3 2019/05/01 15:17:47 mhliu Exp $
 /*
  * Copyright 1996-2014 United States Government as represented by the
  * Administrator of the National Aeronautics and Space Administration.
@@ -646,7 +646,14 @@ synchronized(v) {
 		    for (int i=0; i<dimSizes[0]; i++) {
 			System.arraycopy(_data, n, aByteArray, 0, 
 					 (int)numElements);
-			Array.set(_dataArray, i, new String(aByteArray));
+                        int iend = (int)numElements;
+                        for (int j=0; j<(int)numElements; j++) {
+                          if (aByteArray[j] == 0) {
+                            iend = j;
+                            break;
+                          }
+                        }
+			Array.set(_dataArray, i, new String(aByteArray, 0, iend));
 			n += numElements;
 		    }
 		}
@@ -717,8 +724,14 @@ synchronized(v) {
 			for (int j=0; j<dimSizes[nDims-1]; j++) {
 			    System.arraycopy(_data, n, 
 					     aByteArray, 0, (int)numElements);
-			    ((String [])aRow)[j] = 
-				new String(aByteArray);
+                            int iend = (int)numElements;
+                            for (int k=0; k<(int)numElements; k++) {
+                              if (aByteArray[k] == 0) {
+                                iend = k;
+                                break;
+                              }
+                            }
+			    ((String [])aRow)[j] = new String(aByteArray, 0, iend);
 			    n += numElements;
 			}
 			break;
@@ -794,22 +807,31 @@ synchronized(v) {
 
 	if (nDims <= 1 || numValues == 1)
 	    if ((dataType == CDF_CHAR) || (dataType == CDF_UCHAR)) {
+                int lenx;
 		String dataClassName = _dataArray.getClass().getName();
 		if (dataClassName.indexOf("[B") == -1) {
 		  int n = 0;
 		  if (nDims == 0) {
 		    _data = new byte [(int)numElements];
-		    for (int i=0; i<numElements; i++)	// fill with spaces 
-			((byte [])_data)[i] = 32;
-		    System.arraycopy(((String)_dataArray).getBytes(),
-				     0, _data, 0,
-				     ((String)_dataArray).length());
+		    for (int i=0; i<numElements; i++)	// fill with spaces or 0
+			((byte [])_data)[i] = 0;
+                    if (((String)_dataArray).length() != 0) {
+                      lenx = ((String)_dataArray).length();
+		      System.arraycopy(((String)_dataArray).getBytes(),
+				       0, _data, 0,
+				       (int)((lenx>numElements)?numElements:lenx));
+		    }
 		  } else {
 		    buildDataObject();
 		    for (int i=0; i<dimSizes[0]; i++) {
-			System.arraycopy(((String [])_dataArray)[i].getBytes(),
-					 0, _data, n, 
-					 ((String [])_dataArray)[i].length());
+                        for (int j=0; j<numElements; j++)
+                            ((byte [])_data)[n+j] = 0;
+                        if (((String [])_dataArray)[i].length() != 0) {
+                          lenx = ((String [])_dataArray)[i].length();
+			  System.arraycopy(((String [])_dataArray)[i].getBytes(),
+					   0, _data, n, 
+					   (int)((lenx>numElements)?numElements:lenx));
+			}
 			n += numElements;
 		    }
 		  } 
@@ -881,10 +903,14 @@ synchronized(v) {
 		    case (int)CDF_UCHAR:
 			aRow = (String [])Array.get(subArrays[(int)nDims - 2], i);
 			for (int j=0; j<dimSizes[nDims - 1]; j++) {
-			    if (((String [])aRow)[j] != null)
+                            for (int k=0; k<numElements; k++)
+                                ((byte [])_data)[n+k] = 0;
+			    if (((String [])aRow)[j] != null) {
+                                int lenx = ((String [])aRow)[j].length();
 				System.arraycopy(((String [])aRow)[j].getBytes(),0,
 						 _data,n,
-						 ((String [])aRow)[j].length());
+                                                 (int)((lenx>numElements)?numElements:lenx));
+			    }
 			    n += numElements;
 			}
 			break;
@@ -1091,29 +1117,42 @@ synchronized(v) {
 	case (int)CDF_UCHAR:
 	    dataSignature = "[B";
 	    _data = new byte [(int)numValues];
-	    if (padValue == null)
-	      for (int i=0; i<numValues; i++)
-	        if (dataTypeX == (int)CDF_CHAR)
-		   ((byte [])_data)[i] = 
-			(((new Character(DEFAULT_CHAR_PADVALUE))
-					.toString()).getBytes())[0];
-	        else
-		   ((byte [])_data)[i] = 
-			(((new Character(DEFAULT_UCHAR_PADVALUE))
-					.toString()).getBytes())[0];
-	    else {
+            for (int i=0; i<(int)numValues; i++) ((byte [])_data)[i] = 0;
+	    if (padValue == null) {
+              if (dataTypeX == (int)CDF_CHAR)
+                ((byte [])_data)[0] = 
+                      (((new Character(DEFAULT_CHAR_PADVALUE))
+                                      .toString()).getBytes())[0];
+	      else
+		((byte [])_data)[0] = 
+		      (((new Character(DEFAULT_UCHAR_PADVALUE))
+				      .toString()).getBytes())[0];
+              for (int i=1; i<numValues; i++)
+                  ((byte [])_data)[i] = 0; 
+	    } else {
+              int i, j;
 	      byte[] myBytes = ((String)padValue).getBytes();
 	      int ilen = myBytes.length;
               if (ilen > 0) {
-	        for (int i=0, j=0; i<numValues; i++, j++) {
+                int itotal = ilen;
+                int numElms = (int) _myVar.getNumElements();
+                int numItems;
+                numItems = (int)numValues / numElms;
+                if (itotal > numElms) itotal = numElms;
+                for (j=0; j<numItems; j++) {
+                  for (i=0; i<itotal; i++)
+                    ((byte [])_data)[j*numElms+i] = myBytes[i];
+                }
+//	        for (int i=0, j=0; i<numValues; i++, j++) {
 //		  if (j == numElements) j = 0;
-                  if (j == ilen) j = 0;
-		  ((byte [])_data)[i] = myBytes[j];
-	        }
-              } else { /* nul-string... fill with spaces */
-	        for (int i=0; i<numValues; i++) {
-		  ((byte [])_data)[i] = ((byte[]) (new String(" ").getBytes()))[0];
-	        }
+//                  if (j == ilen) j = 0;
+//		  ((byte [])_data)[i] = myBytes[j];
+//	        }
+//              } else { /* nul-string... fill with spaces */
+//	        for (int i=0; i<numValues; i++) {
+//		  ((byte [])_data)[i] = ((byte[]) (new String(" ").getBytes()))[0];
+//		  ((byte [])_data)[i] = 0;
+//	        }
               } 
 	    }
 	    break;
@@ -1327,13 +1366,16 @@ synchronized(v) {
             if (nDims == 0 || numValues == 1) {
                 if (_myVar.getDataType() == CDF_EPOCH) {
                     Double lEpoch = (Double) _dataArray;
-                    System.out.println (Epoch.encode(lEpoch.doubleValue()));
+                    System.out.println (Epoch.encode4(lEpoch.doubleValue()));
                 } else if (_myVar.getDataType() == CDF_EPOCH16) {
                     double[] lEpoch16 = (double[]) _dataArray;
-                    System.out.println (Epoch16.encode(lEpoch16));
+                    System.out.println (Epoch16.encode4(lEpoch16));
                 } else if (_myVar.getDataType() == CDF_TIME_TT2000) {
                     Long lEpoch = (Long) _dataArray;
                     System.out.println (CDFTT2000.encode(lEpoch.longValue()));
+                } else if (_myVar.getDataType() == CDF_CHAR ||
+                           _myVar.getDataType() == CDF_UCHAR) {
+                    System.out.println ("\""+_dataArray.toString()+"\"");
                 }
                 else
                     System.out.println(_dataArray.toString());
@@ -1346,7 +1388,7 @@ synchronized(v) {
                 case (int)CDF_UCHAR:
                     for (int j=0; j<dimSizes[nDims - 1]; j++) {
                         if (j>0) System.out.print(",");
-                        System.out.print(((String [])_dataArray)[j]);
+                        System.out.print("\""+((String [])_dataArray)[j]+"\"");
                     }
                     break;
                 case (int)CDF_BYTE:
@@ -1431,7 +1473,7 @@ synchronized(v) {
                     for (int j=0; j<dimSizes[nDims - 1]; j++) {
                         if (j>0) System.out.print(",");
 			System.out.print(
-                                   Epoch.encode(((double [])_dataArray)[j]));
+                                   Epoch.encode4(((double [])_dataArray)[j]));
                     }
                     break;
                 case (int)CDF_EPOCH16:
@@ -1440,7 +1482,7 @@ synchronized(v) {
                         if (j>0) System.out.print(",");
                         tmp[0] = ((double [])_dataArray)[2*j];
                         tmp[1] = ((double [])_dataArray)[2*j+1];
-                        System.out.print(Epoch16.encode(tmp));
+                        System.out.print(Epoch16.encode4(tmp));
                     }
                     break;
                 default:
@@ -1516,7 +1558,7 @@ synchronized(v) {
                             (String [])Array.get(subArrays[(int)nDims - 2], i);
                         for (int j=0; j<dimSizes[nDims - 1]; j++) {
                             if (j>0) System.out.print(",");
-                            System.out.print(((String [])aRow)[j]);
+                            System.out.print("\""+((String [])aRow)[j]+"\"");
                         }
                         break;
                     case (int)CDF_BYTE:
@@ -1610,7 +1652,7 @@ synchronized(v) {
                         for (int j=0; j<dimSizes[nDims - 1]; j++) {
                             if (j>0) System.out.print(",");
 			    System.out.print(
-                                       Epoch.encode(((double [])aRow)[j]));
+                                       Epoch.encode4(((double [])aRow)[j]));
                         }
                         break;
                     case (int)CDF_EPOCH16:
@@ -1620,7 +1662,7 @@ synchronized(v) {
                             if (j>0) System.out.print(",");
                             tmp[0] = ((double [])aRow)[2*j];
                             tmp[1] = ((double [])aRow)[2*j+1];
-                            System.out.print(Epoch16.encode(tmp));
+                            System.out.print(Epoch16.encode4(tmp));
                         }
                         break;
                     default:
