@@ -59,9 +59,15 @@ public class Entry extends Object implements CDFObject, CDFConstants {
     /**
      * The number of elements for this entry
      */
-    private long numElements;
+    private long numElements = -1;
     private long xnumElements;   // a temp one
 
+    /**
+     * The number of strings for this entry
+     */
+    private long numStrings;
+    private long xnumStrings;   // a temp one
+
     /**
      * The entry data
      */
@@ -113,6 +119,7 @@ synchronized(myAttribute) {
 	Vector cmds = new Vector();
 	Vector item0 = new Vector();
 	Vector item1 = new Vector();
+	Vector item2 = new Vector();
         Vector itemA = new Vector();
 	Vector itemB = new Vector();
 	Vector itemC = new Vector(); 
@@ -145,6 +152,12 @@ synchronized(myAttribute) {
             item1.addElement("numElements");
             item1.addElement("J");
           cmds.addElement(item1);
+          if (theEntry.scope != GLOBAL_SCOPE) {
+            cmds.addElement(new Long(zENTRY_NUMSTRINGS_));
+              item2.addElement("numStrings");
+              item2.addElement("J");
+            cmds.addElement(item2);
+          }
         cmds.addElement(new Long(NULL_));
 
 	theEntry.cdfID = myAttribute.getMyCDF().getID();
@@ -158,7 +171,10 @@ synchronized(myAttribute) {
 	long status = theEntry.myAttribute.getMyCDF().getStatus();
         if (status < CDF_WARN) throw new CDFException(status);
 */
-
+        if (theEntry.scope == VARIABLE_SCOPE && 
+            (theEntry.dataType == CDF_CHAR || theEntry.dataType == CDF_UCHAR)) {
+          if (theEntry.numStrings < 1) theEntry.numStrings = 1;
+        }
         /********************/
 	/* Get the data out */
         /********************/
@@ -195,7 +211,6 @@ synchronized(myAttribute) {
 
 	theEntry.myAttribute.getMyCDF().
 	    executeCommand((CDFObject)theEntry, cmds);
-
 	myAttribute.addEntry(theEntry, (int)id);
 	return theEntry;
     }
@@ -250,6 +265,9 @@ synchronized(myAttribute) {
      *                    <LI>CDF_TIME_TT2000  - 8-byte, signed integer
      *                 </UL>
      * @param data the entry data to be added <P>
+     * Note: From CDF V3.7.0, the string typed data of variable entry can 
+     *       be an array of strings. Data from previous versions can only
+     *       be a single string.
      *
      * @return newly created attribute entry <P>
      * @throws CDFException CDFException if there is a problem creating an entry
@@ -281,14 +299,16 @@ synchronized(myAttribute) {
 
 	Entry theEntry         = new Entry(myAttribute, id);
 	theEntry.dataType      = dataType;
+/*
 	theEntry.data          = data;
 	theEntry.dataSignature = CDFUtils.getSignature(data);
-	theEntry.numElements   = CDFUtils.getNumElements(dataType, data);
-
+        theEntry.numElements   = CDFUtils.getNumElements(dataType, data);
+*/
 	Vector cmds  = new Vector();
 	Vector item0 = new Vector();
 	Vector item1 = new Vector();
 	Vector item2 = new Vector();
+	Vector item3 = new Vector();
 	Vector itemA = new Vector();
 	Vector itemB = new Vector();
 	Vector itemC = new Vector();
@@ -308,6 +328,26 @@ synchronized(myAttribute) {
             itemC.addElement("J");
           cmds.addElement(itemC);
 
+          if (theEntry.scope == VARIABLE_SCOPE &&
+              (dataType == CDF_CHAR || dataType == CDF_UCHAR)) {
+            if (CDFUtils.getSignature(data).indexOf("[") != -1) {
+              theEntry.numStrings    = (long) ((String[])data).length;
+              theEntry.data          = (Object) CDFUtils.mergeFromStrings(data);
+              theEntry.numElements   = (long) ((String)(theEntry.data)).length();
+              theEntry.dataSignature = CDFUtils.getSignature(theEntry.data);
+            } else {
+              theEntry.data          = data;
+	      theEntry.numElements   = CDFUtils.getNumElements(dataType, data);
+              theEntry.numStrings    = (long) CDFUtils.getNumStrings(data);
+              theEntry.dataSignature = CDFUtils.getSignature(theEntry.data);
+            }
+          } else {
+            theEntry.data          = data;
+	    theEntry.numElements   = CDFUtils.getNumElements(dataType, data);
+            theEntry.numStrings    = 1;
+            theEntry.dataSignature = CDFUtils.getSignature(theEntry.data);
+          }
+
 	cmds.addElement(new Long(PUT_));
 	  cmds.addElement(((theEntry.scope == GLOBAL_SCOPE) ?
 			   new Long(gENTRY_DATA_) : new Long(zENTRY_DATA_)));
@@ -425,8 +465,7 @@ synchronized(myAttribute) {
 	    } /* switch (dataType) */
 	}
     }
-	    
-    
+
     /**
      * Deletes this entry.
      *
@@ -479,6 +518,15 @@ synchronized(myAttribute) {
 	return dataType;
     }
     
+    /**
+     * Gets the scope of the attribute this entry in in.
+     *
+     * @return the scope of this entry
+     */
+    public synchronized long getScope() {
+	return scope;
+    }
+    
     /**
      * Gets the number of elements in this entry.  For CDF_CHAR, it 
      * returns the number of characters stored.  
@@ -498,20 +546,45 @@ synchronized(myAttribute) {
 	return numElements;
     }
 
+    /**
+     * Gets the number of strings in the data. If the data is not string
+     * type, 0 is returned. For global attribute entry, it always returns 1.
+     *
+     * @return the string number for this entry 
+     */
+    public synchronized long getNumStrings() {
+
+        if (dataType == CDF_CHAR || dataType == CDF_UCHAR) {
+          if (scope == VARIABLE_SCOPE) return numStrings;
+          else return 1L;
+        } else
+          return 0L;
+    }
+ 
     /**
      * Gets the data for this entry. Build a new data object so a duplicated
      * variable will have its own copy of the entry data.
      *
-     * @return the data for this entry 
+     * @return the data for this entry
+     * Note: From CDF V3.7.0, the string typed data of variable entry can 
+     *       return an array of strings. Data from previous versions only
+     *       returns a single string.
      *
      * @throws CDFException An exception is thrown if the operation fails
+     *
      */
     public synchronized Object getData() throws CDFException {
 
       try {
 	if ((dataType == CDF_CHAR) || (dataType == CDF_UCHAR)) {
-	    String data2 = new String(((String)data).toString());
-            return data2;
+            if (scope == GLOBAL_SCOPE)
+	      return new String((String)data.toString());
+            else {
+              if (numStrings < 2)
+                return new String((String)data.toString());
+              else
+                return CDFUtils.breakIntoStrings((String)data.toString());
+            }
 	} else {
 	  switch ((int)dataType) {
 	    case (int)CDF_BYTE:
@@ -730,6 +803,9 @@ synchronized(myAttribute) {
      *
      * @param  dataType the data type of the entry data
      * @param  data the entry data to write
+     * Note: From CDF V3.7.0, the string typed data of variable entry can 
+     *       be an array of strings. Data from previous versions can only
+     *       be a single string.
      * @throws CDFException An exception is thrown if the operation fails
      *
      */
@@ -760,6 +836,26 @@ synchronized(myAttribute) {
               itemC.addElement("J");
             cmds.addElement(itemC);
             
+          if (this.scope == VARIABLE_SCOPE &&
+              (dataType == CDF_CHAR || dataType == CDF_UCHAR)) {
+            if (CDFUtils.getSignature(data).indexOf("[") != -1) {
+              this.numStrings    = (long) ((String[])data).length;
+              this.data         = (Object) CDFUtils.mergeFromStrings(data);
+              this.numElements   = (long) ((String)(this.data)).length();
+              this.dataSignature = CDFUtils.getSignature(this.data);
+            } else {
+              this.data         = data;
+              this.numElements   = CDFUtils.getNumElements(dataType, data);
+              this.numStrings    = (long) CDFUtils.getNumStrings(data);
+              this.dataSignature = CDFUtils.getSignature(this.data);
+            }
+          } else {
+            this.data         = data;
+            this.numElements   = CDFUtils.getNumElements(dataType, data);
+            this.numStrings    = 1;
+            this.dataSignature = CDFUtils.getSignature(this.data);
+          }
+
           cmds.addElement(new Long(PUT_));
             cmds.addElement(((scope == GLOBAL_SCOPE) ?
                              new Long(gENTRY_DATA_) :
@@ -779,6 +875,7 @@ synchronized(myAttribute) {
           id = getID();
           this.xdataType = this.dataType;
           this.xnumElements = this.numElements;
+          this.xnumStrings = this.numStrings;
           this.xdataSignature = this.dataSignature;
           this.xdata = this.data;
           this.dataType = dataType;
