@@ -1,4 +1,4 @@
-// $Id: Variable.java,v 1.1.1.1 2017/01/04 20:58:09 liu Exp $
+// $Id: Variable.java,v 1.1.1.1 2018/08/29 19:16:17 mhliu Exp $
 /*
  * Copyright 1996-2014 United States Government as represented by the
  * Administrator of the National Aeronautics and Space Administration.
@@ -388,12 +388,13 @@ synchronized(myCDF) {
     {
 
 synchronized(myCDF) {
+        if (varName == null || varName.trim().length() == 0)
+            throw new CDFException(BAD_VAR_NAME);
+
 	long idx = myCDF.getVariableID(varName);
 	if (idx != -1) 
 	    throw new CDFException(VAR_EXISTS);
 
-	if (varName == null || varName.length() == 0) 
-	    throw new CDFException(BAD_VAR_NAME);
         Variable v;
 
 	Vector cmds = new Vector();
@@ -2266,13 +2267,15 @@ synchronized(myCDF) {
 	Vector itemB = new Vector();
 
 	if (this.dataType == CDF_CHAR || this.dataType == CDF_UCHAR) {
+/*
 	  int len = ((String)padValue).length();
-	  if  (len < this.numElements) { /* Fill space(s) */
+	  if  (len < this.numElements) {
 	    StringBuffer tmp = new StringBuffer((String)padValue);
             int ix;
             for (ix = 0; ix < (this.numElements - len); ++ix) tmp.append(' ');
 	    this.padValue = new String(tmp.toString());
 	  } else
+*/
 	    this.padValue = padValue;
 	} else
 	  this.padValue = padValue;
