@@ -1,4 +1,4 @@
-// $Id: CDFUtils.java,v 1.1.1.1 2017/01/04 20:58:09 liu Exp $
+// $Id: CDFUtils.java,v 1.1.1.1 2018/08/29 19:16:17 mhliu Exp $
 /*
  * Copyright 1996-2014 United States Government as represented by the
  * Administrator of the National Aeronautics and Space Administration.
@@ -141,6 +141,33 @@ public class CDFUtils implements CDFConstants {
 	}
     }
 
+    /**
+     * Gets the number of strings contained in the given data object.  <P>
+     *
+     * @param data the data object to be examined <P> 
+     *
+     * @return The number of strings if the object is of string type and the
+     *         data has the predefined STRINGDELIMITER (as "\\N ") delimiter
+     *         in it. Or, 0 if non-string datat type. 
+     */
+    public static long getNumStrings(Object data) {
+        String signature = CDFUtils.getSignature(data).toLowerCase();
+        if (signature.charAt(0) == '[' &&
+            signature.indexOf("string") != -1) {
+          return ((String[])data).length;
+        } else if (signature.indexOf("string") != -1) {
+            long numStrings = 1;
+            int ix;
+            StringBuffer tmp = new StringBuffer((String)data.toString());
+            while ((ix = tmp.indexOf(STRINGDELIMITER)) != -1) {
+              tmp = new StringBuffer(tmp.substring(ix+STRINGDELIMITER.length()));
+              ++numStrings;
+            }
+            return numStrings;
+	} else 
+          return 0L;
+    }
+
     /**
      *  Prints the value of the given data on the screen.  Data can be a 
      *  java primitive data type, Java Object (non-array), or 
@@ -151,7 +178,7 @@ public class CDFUtils implements CDFConstants {
      */
     public static void printData (Object data) {
 
-	printData(data, 0, false);
+	CDFUtils.printData(data, 0, true);
     }
 
     /**
@@ -167,6 +194,25 @@ public class CDFUtils implements CDFConstants {
      */
     public static void printData (Object data, int which, boolean iso8601) {
 
+        CDFUtils.printData(data, 0, true, "");
+    }
+
+    /**
+     *  Prints the value of the given data on the screen.  Data can be a
+     *  java primitive data type, Java Object (non-array), or
+     *  1-dimensional array of primitive Java data type.
+     *
+     *  @param data the data to be printed
+     *  @param which the Epoch data type data indicator
+     *               1 if CDF_EPOCH, 2 if CDF_EPOCH16 or 3 if CDF_TIME_TT2000
+     *  @param iso8601 the ISO 8601 indicator for EPOCH data
+     *  @param addZ Added the passed string to the end, (likely a "Z", to
+     *              all CDF epoch data types, not just CDF_TIME_TT2000)
+     *
+     */
+    public static void printData (Object data, int which, boolean iso8601,
+                                  String addZ) {
+
         int  i, arrayLength, jj = 1;
         String signature = CDFUtils.getSignature(data);
 	if (which == 2) jj = 2;
@@ -191,39 +237,43 @@ public class CDFUtils implements CDFConstants {
                                          (i==(arrayLength-jj)?"":" "));
                      else
                        System.out.print (CDFTT2000.encode(Array.getLong(data,i))+
-                                         (i==(arrayLength-jj)?"":" "));
+                                         addZ+(i==(arrayLength-jj)?"":" "));
 
                  else if (signature.charAt(1) == 'F')
                      System.out.print (Array.getFloat(data,i)+
                                        (i==(arrayLength-jj)?"":" "));
 
-                 else if (signature.charAt(1) == 'D')
+                 else if (signature.charAt(1) == 'D') {
 		   if (which == 1) {
-		     System.out.print ((!iso8601)?
-                                       Epoch.encode(Array.getDouble(data,i))+
-                                        (i==(arrayLength-jj)?"":" "):
-                                       Epoch.encode4(Array.getDouble(data,i))+
-                                        (i==(arrayLength-jj)?"":" "));
+		     System.out.print (((!iso8601)?
+                                        Epoch.encode(Array.getDouble(data,i)):
+                                        (Epoch.encode4(Array.getDouble(data,i))+
+                                         addZ))+(i==(arrayLength-jj)?"":" "));
                    } else if (which == 2) {
                      double[] mmm = new double[2];
                      mmm[0] = Array.getDouble(data,i);
                      mmm[1] = Array.getDouble(data,i+1);
-                     System.out.print ((!iso8601)?
-                                       Epoch16.encode(mmm):
-                                       Epoch16.encode4(mmm)+
-                                        (arrayLength==2?"":" "));
+                     System.out.print (((!iso8601)?
+                                        Epoch16.encode(mmm):
+                                        (Epoch16.encode4(mmm)+
+                                         addZ))+(arrayLength==2?"":" "));
 		   } else
                      System.out.print (Array.getDouble(data,i)+
                                        (i==(arrayLength-jj)?"":" "));
+                 } else {
+                     if (i>0) System.out.print(" \"");
+                     System.out.print (Array.get((String[])data, i));
+                     if (i<(arrayLength-1)) System.out.print("\" ");
+                 }
             }
         }
         else {
 	   if (which == 1) 
              System.out.print ((!iso8601)?
                                Epoch.encode(((Double)data).doubleValue()):
-                               Epoch.encode4(((Double)data).doubleValue()));
+                               Epoch.encode4(((Double)data).doubleValue())+addZ);
            else if (which == 3)
-             System.out.print (CDFTT2000.encode(((Long)data).longValue()));
+             System.out.print (CDFTT2000.encode(((Long)data).longValue())+addZ);
            else
              System.out.print (data);
 	}
@@ -270,7 +320,8 @@ public class CDFUtils implements CDFConstants {
      *                   of the object/data is printed as a text-output stream
      */
     public static void printData (Object data, PrintWriter outWriter) {
-	printData(data, outWriter, 0, false);
+
+	CDFUtils.printData(data, outWriter, 0, false);
     }
 
     /**
@@ -287,7 +338,7 @@ public class CDFUtils implements CDFConstants {
      */
     public static void printData (Object data, PrintWriter outWriter,
 				  int which) {
-        printData(data, outWriter, which, false);
+        CDFUtils.printData(data, outWriter, which, false);
     }
 
     /**
@@ -306,6 +357,28 @@ public class CDFUtils implements CDFConstants {
     public static void printData (Object data, PrintWriter outWriter,
 				  int which, boolean iso8601) {
 
+        CDFUtils.printData(data, outWriter, which, iso8601, "");
+    }
+
+    /**
+     *  Prints the value of the given data to the place designated by
+     *  PrintWriter that can be a file, Sysem.out, System.err, and etc.  
+     *  Data can be a java primitive data type, Java Object (non-array), or 
+     *  1-dimensional array of primitive Java data type. <P>
+     *
+     *  @param data the data to be printed <P>
+     *  @param outWriter the print writer to which formatted representations 
+     *                   of the object/data is printed as a text-output stream
+     *  @param which the Epoch data type data indicator
+     *               1 if CDF_EPOCH, 2 if CDF_EPOCH16 or 3 if CDF_TIME_TT2000
+     *  @param iso8601 the ISO 8601 indicator for EPOCH data
+     *  @param addZ Added the passed string to the end of an ISO8601 format
+     *              string, (likely a "Z", to all CDF epoch data types, not 
+     *              just CDF_TIME_TT2000).
+     */
+    public static void printData (Object data, PrintWriter outWriter,
+				  int which, boolean iso8601, String addZ) {
+
         int  i, arrayLength, jj = 1;
         String signature = CDFUtils.getSignature(data);
 	if (which == 2) jj = 2;
@@ -326,7 +399,8 @@ public class CDFUtils implements CDFConstants {
                      if (which == 0)
                        outWriter.print (Array.getLong(data,i)+" ");
                      else
-                       outWriter.print (CDFTT2000.encode(Array.getLong(data,i))+" ");
+                       outWriter.print (CDFTT2000.encode(Array.getLong(data,i))+
+                                        addZ+" ");
 
                  else if (signature.charAt(1) == 'F')
                      outWriter.print (Array.getFloat(data,i)+" ");
@@ -335,16 +409,16 @@ public class CDFUtils implements CDFConstants {
                      if (which == 1) {
                        outWriter.print (((!iso8601)?
                                          Epoch.encode(Array.getDouble(data,i)):
-                                         Epoch.encode4(Array.getDouble(data,i)))+
-                                        " ");
+                                         (Epoch.encode4(Array.getDouble(data,i))+
+                                          addZ))+" ");
                      } else if (which == 2) {
                        double[] mmm = new double[2];
                        mmm[0] = Array.getDouble(data,i);
                        mmm[1] = Array.getDouble(data,i+1);
                        outWriter.print (((!iso8601)?
                                          Epoch16.encode(mmm):
-                                         Epoch16.encode4(mmm))+
-                                         (arrayLength==2?"":" "));
+                                         (Epoch16.encode4(mmm)+
+                                          addZ))+(arrayLength==2?"":" "));
                      } else
                        outWriter.print (Array.getDouble(data,i)+" ");
 		 }
@@ -352,11 +426,11 @@ public class CDFUtils implements CDFConstants {
         }
         else {
 	   if (which == 1)
-	     outWriter.print (((!iso8601)?
+	     outWriter.print ((!iso8601)?
                                Epoch.encode(((Double)data).doubleValue()):
-                               Epoch.encode4(((Double)data).doubleValue())));
+                               (Epoch.encode4(((Double)data).doubleValue())+addZ));
            else if (which == 3)
-             outWriter.print (CDFTT2000.encode(((Long)data).longValue()));
+             outWriter.print (CDFTT2000.encode(((Long)data).longValue())+addZ);
            else
 	     outWriter.print (data);
 	}
@@ -379,6 +453,59 @@ public class CDFUtils implements CDFConstants {
 	return CDFUtils.getStringData(data, " ", 0);
     }
 
+    /**
+     *  Merge an array of strings into a single string with "\ N ",
+     *  a 3-char separator. <P>
+     *
+     *  @param data the strings to be merged. <P>
+     *  @return The merged string. <BR>
+     *          If the input is any single string, then it returns as is.
+     *
+     */
+    public static String mergeFromStrings (Object data) {
+
+        long numStrings = CDFUtils.getNumStrings(data);
+        if (numStrings > 1) {
+          StringBuffer tmp = new StringBuffer();
+          int ix;
+          for (ix = 0; ix < numStrings; ++ix) {
+            tmp.append(((String[])data)[ix]);
+            if (ix != (numStrings-1)) tmp.append(STRINGDELIMITER);
+          }
+          return tmp.toString();
+        } else
+          return (String)data;
+    }
+
+    /**
+     *  Break down a merged string into their original array of strings. 
+     *  This is a reversed method of MergeStrings. The mergeed string uses
+     *  "\ N ", a 3-char, as the separator. <P>
+     *
+     *  @param data the merged string to be broken down. <P>
+     *  @return An array of strings. <BR>
+     *          If the input string has no separator, it is a single string.
+     *          The returned array will have only one element.
+     *
+     */
+    public static String[] breakIntoStrings (String data) {
+
+        long numStrings = CDFUtils.getNumStrings(data);
+        String[] data2 = new String[(int)numStrings];
+        int ix;
+        StringBuffer tmp = new StringBuffer(data);
+        for (ix = 0; ix < numStrings; ++ix) {
+           int dstr = tmp.indexOf(STRINGDELIMITER);
+           if (dstr != -1) {
+             data2[ix] = tmp.substring(0, dstr).toString();
+             tmp = new StringBuffer(tmp.substring(dstr+STRINGDELIMITER.length()));
+           } else {
+             data2[ix] = new String(tmp);
+           }
+        }
+        return data2;
+    }
+
     /**
      *  Returns the string value of the given data.
      *  Data can be a java primitive
@@ -430,7 +557,31 @@ public class CDFUtils implements CDFConstants {
      *          the user defined separator. 
      *
      */
-    public static String getStringData (Object data, String separator, int epochType) {
+    public static String getStringData (Object data, String separator,
+                                        int epochType) {
+
+        return CDFUtils.getStringData(data, separator, 0, "");
+    }
+    /**
+     *  returns the string of the value of the given data.  
+     *  Data can be a java primitive
+     *  data type, Java Object (non-array), or 1-dimensional array of
+     *  primitive Java data type. <P>
+     *
+     *  @param data the data to be parsed <P>
+     *  @param separator the delimiter for array elements <P>
+     *  @param epochType Epoch or Epoch16 data type indicator <BR>
+     *         == 1 for EPOCH, == 2 for EPOCH16, == 3 for TT2000, 
+     *         == 0 for other data types <P>
+     *  @param addZ Added the passed string to the end, likely a "Z", to
+     *              CDF_TIME_TT2000 data type
+     *  @return The string value of the given data/object. <BR>
+     *          If the data is an array, its elements are delimited by 
+     *          the user defined separator. 
+     *
+     */
+    public static String getStringData (Object data, String separator,
+                                        int epochType, String addZ) {
         int  i, arrayLength, jj;
         String signature = CDFUtils.getSignature(data);
 	StringBuffer buffer = new StringBuffer();
@@ -494,7 +645,7 @@ public class CDFUtils implements CDFConstants {
                    if (epochType == 0)
                      buffer.append(((Long)data).toString());
                    else
-                     buffer.append(CDFTT2000.encode(((Long)data).longValue()));
+                     buffer.append(CDFTT2000.encode(((Long)data).longValue())+addZ);
 
                  else if (signature.indexOf("Float") != -1)
                    buffer.append(((Float)data).toString());
@@ -870,6 +1021,11 @@ public class CDFUtils implements CDFConstants {
      *             <LI>ALPHAVMSd_ENCODING
      *             <LI>ALPHAVMSg_ENCODING
      *             <LI>ALPHAVMSi_ENCODING
+     *             <LI>ARM_LITTLE_ENCODING
+     *             <LI>ARM_BIG_ENCODING
+     *             <LI>IA64VMSi_ENCODING
+     *             <LI>IA64VMSd_ENCODING
+     *             <LI>IA64VMSg_ENCODING
      *          </UL><P>
      *
      *  @return string representation of encodingType.
@@ -889,6 +1045,11 @@ public class CDFUtils implements CDFConstants {
      *             <LI>ALPHAVMSd
      *             <LI>ALPHAVMSg
      *             <LI>ALPHAVMSi
+     *             <LI>ARM_LITTLE
+     *             <LI>ARM_BIG
+     *             <LI>IA64VMSi
+     *             <LI>IA64VMSd
+     *             <LI>IA64VMSg
      *             <LI>UNKNOWN (for unknown encodingType)
      *          </UL>
      */
@@ -901,10 +1062,12 @@ public class CDFUtils implements CDFConstants {
                                " ",          "HP",
                                "NeXT",       "ALPHAOSF1",  
                                "ALPHAVMSd",  "ALPHAVMSg",  
-                               "ALPHAVMSi"}; 
+                               "ALPHAVMSi",  "ARM_LITTLE",
+                               "ARM_BIG",    "IA64VMSi",
+                               "IA64VMSd",   "IA64VMSg"}; 
 
         if ((encodingType == 0L) || (encodingType == 10L) || 
-            (encodingType > ALPHAVMSi_ENCODING))
+            (encodingType > IA64VMSg_ENCODING))
 
             return "UNKNOWN";
 
@@ -932,6 +1095,11 @@ public class CDFUtils implements CDFConstants {
      *             <LI>ALPHAVMSd
      *             <LI>ALPHAVMSg
      *             <LI>ALPHAVMSi
+     *             <LI>ARM_LITTLE
+     *             <LI>ARM_BIG
+     *             <LI>IA64VMSi
+     *             <LI>IA64VMSd
+     *             <LI>IA64VMSg
      *          </UL><P>
      * 
      *  @return long representation of encodingType.  
@@ -951,6 +1119,11 @@ public class CDFUtils implements CDFConstants {
      *             <LI>ALPHAVMSd_ENCODING
      *             <LI>ALPHAVMSg_ENCODING
      *             <LI>ALPHAVMSi_ENCODING
+     *             <LI>ARM_LITTLE_ENCODING
+     *             <LI>ARM_BIG_ENCODING
+     *             <LI>IA64VMSi_ENCODING
+     *             <LI>IA64VMSd_ENCODING
+     *             <LI>IA64VMSg_ENCODING
      *             <LI>-1 (for unknown encodingType)
      *          </UL>
      */
@@ -963,7 +1136,9 @@ public class CDFUtils implements CDFConstants {
                                " ",          "HP",
                                "NeXT",       "ALPHAOSF1",
                                "ALPHAVMSd",  "ALPHAVMSg",
-                               "ALPHAVMSi"};
+                               "ALPHAVMSi",  "ARM_LITTLE",
+                               "ARM_BIG",    "IA64VMSi",
+                               "IA64VMSd",   "IA64VMSg"};
 
 	if ("".equals(encodingType) || encodingType.equals(" "))
 	   return -1;
@@ -1011,6 +1186,11 @@ public class CDFUtils implements CDFConstants {
      *             <LI>ALPHAVMSd_DECODING
      *             <LI>ALPHAVMSg_DECODING
      *             <LI>ALPHAVMSi_DECODING
+     *             <LI>ARM_LITTLE_DECODING
+     *             <LI>ARM_BIG_DECODING
+     *             <LI>IA64VMSi_DECODING
+     *             <LI>IA64VMSd_DECODING
+     *             <LI>IA64VMSg_DECODING
      *             <LI>-1 (for unknown encodingType)
      *          </UL><P>
      *  @return string representation of decodingType. See
