@@ -1,4 +1,4 @@
-// $Id: Epoch.java,v 1.1.1.1 2017/01/04 20:58:09 liu Exp $
+// $Id: Epoch.java,v 1.16 2019/04/11 14:04:26 mhliu Exp $
 /*
  * Copyright 1996-2014 United States Government as represented by the
  * Administrator of the National Aeronautics and Space Administration.
@@ -25,7 +25,7 @@ import java.text.*;
  * // Get the milliseconds to Aug 5, 1990 at 5:00
  * double ep = Epoch.compute(1990, 8, 5, 5, 0, 0, 0);
  * //Get the year, month, day, hour, minutes, seconds, milliseconds for ep
- * long times[] = Epoch.breakdown(ep);
+ * long[] times = Epoch.breakdown(ep);
  * for (int i=0;i&lt;times.length;i++)
  *     System.out.print(times[i]+" ");
  * System.out.println();
@@ -61,6 +61,172 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 	"Dec"
     };
 
+    /**
+     * 
+     * This function parses an array of date/time strings and returns an array 
+     * of EPOCH values.  The string format must be exactly as one of the
+     * followings (which are made from the corresponding encoding methods).
+     *
+     * <PRE>
+     * Style 0 - 
+     * Format:   dd-mmm-yyyy hh:mm:ss.mmm
+     * Examples:  1-Apr-1990 03:05:02.000
+     *           10-Oct-1993 23:45:49.999
+     *
+     * Style 1 - 
+     * Format:   yyyymmdd.ttttttt
+     * Examples: 19900401.3658893
+     *           19931010.0000000
+     *
+     * Style 2 - 
+     * Format:   yyyymmddhhmmss
+     * Examples: 19900401030502
+     *           19931010234549
+     *
+     * Style 3 - 
+     * Format:   yyyy-mm-ddThh:mm:ss.mmmZ
+     * Examples: 1990-04-01T03:05:02.000Z
+     *           1993-10-10T23:45:49.999Z
+     *
+     * Style 4 - 
+     * Format:   yyyy-mm-ddThh:mm:ss.mmm
+     * Examples: 1990-04-01T03:05:02.000
+     *           1993-10-10T23:45:49.999
+     *
+     * </PRE>
+     *
+     * @param inStrings the epochs in string representation
+     *
+     * @return the values of the epoch represented by inStrings or null if null
+     *         or an empty input array is detected
+     *
+     * @exception CDFException if a bad epoch value is passed in inStrings
+     */
+    public static double[] toParse (String[] inStrings)
+        throws CDFException
+    {
+        if (inStrings == null) return null;
+        double[] dateTimes = new double[inStrings.length];
+        int ix;
+        for (ix = 0; ix < inStrings.length; ++ix)
+          dateTimes[ix] = toParse (inStrings[ix]);
+        return dateTimes;
+/*
+        String dateTime = inStrings[0].trim();
+        int len = dateTime.length();
+        char aC;
+        if (len == EPOCH_STRING_LEN) { // style 0 or style 3
+          if (dateTime.charAt(11) == ' ') return parse(inStrings);
+          aC = dateTime.charAt(10);
+          if (aC == 'T' || aC == 't' || aC == ' ' ||
+              aC == '/') return parse3(inStrings);
+          else return null;
+        } else if (len == EPOCH4_STRING_LEN) {
+          aC = dateTime.charAt(10);
+          if (aC == 'T' || aC == 't' || aC == ' ' ||
+              aC == '/') return parse4(inStrings);
+          else return null;
+        } else if (len == EPOCH1_STRING_LEN &&
+                   dateTime.matches("\\d+.\\d+")) {
+          return parse1(inStrings);
+        } else if (len == EPOCH2_STRING_LEN &&
+                   dateTime.matches("\\d+")) {
+          return parse2(inStrings);
+//        } else if (len == (EPOCH_STRING_LEN+1) && (dateTime.endsWith("Z") ||
+//                                                   dateTime.endsWith("z")) &&
+//                   dateTime.charAt(11) == ' ') { // style 0 (sort of)
+//          return parse(inStrings);
+        } else if (dateTime.charAt(11) == ' ') {
+          return parse(inStrings);
+        } else if (dateTime.charAt(10) == 'T' || dateTime.charAt(10) == 't' ||
+                   dateTime.charAt(10) == ' ' ||
+                   dateTime.charAt(10) == '/') {
+            if (dateTime.endsWith("Z") || dateTime.endsWith("Z"))
+              return parse3(inStrings);
+            else
+              return parse4(inStrings);
+        } else
+          return null;
+*/
+    }
+
+    /**
+     * 
+     * This function parses a date/time string and returns an EPOCH value. 
+     * The string format must be exactly as one of the
+     * followings (which are made from the corresponding encoding methods).
+     *
+     * <PRE>
+     * Style 0 - 
+     * Format:   dd-mmm-yyyy hh:mm:ss.mmm
+     * Examples:  1-Apr-1990 03:05:02.000
+     *           10-Oct-1993 23:45:49.999
+     *
+     * Style 1 - 
+     * Format:   yyyymmdd.ttttttt
+     * Examples: 19900401.3658893
+     *           19931010.0000000
+     *
+     * Style 2 - 
+     * Format:   yyyymmddhhmmss
+     * Examples: 19900401030502
+     *           19931010234549
+     *
+     * Style 3 - 
+     * Format:   yyyy-mm-ddThh:mm:ss.mmmZ
+     * Examples: 1990-04-01T03:05:02.000Z
+     *           1993-10-10T23:45:49.999Z
+     *
+     * Style 4 - 
+     * Format:   yyyy-mm-ddThh:mm:ss.mmm
+     * Examples: 1990-04-01T03:05:02.000
+     *           1993-10-10T23:45:49.999
+     *
+     * </PRE>
+     *
+     * @param inString the epoch in string representation
+     *
+     * @return the value of the epoch represented by inStrings or null if null
+     *         or an empty input array is detected
+     *
+     * @exception CDFException if a bad epoch value is passed in inStrings
+     */
+    public static double toParse (String inString)
+        throws CDFException
+    {
+        if (inString == null) return ILLEGAL_EPOCH_VALUE;
+        inString = inString.trim();
+        int len = inString.length();
+        char aC;
+        if (len == EPOCH_STRING_LEN) { // style 0 or style 3
+          if (inString.charAt(11) == ' ') return parse(inString);
+          aC = inString.charAt(10);
+          if (aC == 'T' || aC == 't' || aC == ' ' || aC == '/')
+            return parse3(inString);
+          else return ILLEGAL_EPOCH_VALUE;
+        } else if (len == EPOCH4_STRING_LEN) { // style 4
+          aC = inString.charAt(10);
+          if (aC == 'T' || aC == 't' || aC == ' ' || aC == '/')
+            return parse4(inString);
+          else return ILLEGAL_EPOCH_VALUE;
+        } else if (len <= EPOCH1_STRING_LEN && inString.charAt(8) == '.' &&
+                   inString.matches("\\d+.\\d+")) { // style 1
+          return parse1(inString);
+        } else if (len == EPOCH2_STRING_LEN &&
+                   inString.matches("\\d+")) { // style 2
+          return parse2(inString);
+        } else if (inString.charAt(11) == ' ') {
+          return parse(inString);
+        } else if (inString.charAt(10) == 'T' || inString.charAt(10) == 't' ||
+                   inString.charAt(10) == ' ' || inString.charAt(10) == '/') {
+            if (inString.endsWith("Z") || inString.endsWith("z"))
+              return parse3(inString);
+            else
+              return parse4(inString);
+        } else
+          return ILLEGAL_EPOCH_VALUE;
+    }
+
     /**
      * 
      * This function parses an input date/time string and returns an EPOCH
@@ -93,7 +259,9 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 	    second = 0, 
 	    msec   = 0;
 	String monthStr, secondStr;
-
+        inString = inString.trim();
+        if (inString.endsWith("Z") || inString.endsWith("z"))
+            inString = inString.substring(0, inString.length()-1);
 	try {
 
   	    StringTokenizer st = new StringTokenizer(inString, " ");
@@ -122,8 +290,8 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 	    secondStr = st.nextToken();
 	    st = new StringTokenizer(secondStr, ".");
 	    second = Long.parseLong(st.nextToken());
-	    msec   = Long.parseLong(st.nextToken());
-	    
+            if (st.hasMoreTokens())
+ 	      msec   = Long.parseLong(st.nextToken());
 	    return compute(year, month, day, hour, minute, second, msec);
 	} catch (Exception e) {
 	    throw new CDFException(ILLEGAL_EPOCH_FIELD);
@@ -186,6 +354,7 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
     public static double parse1 (String inString)
 	throws CDFException
     {
+        inString = inString.trim();
 	StringTokenizer st = new StringTokenizer(inString, ".");
         try {
 	  String date = st.nextToken();
@@ -197,6 +366,12 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 	    year   = Long.parseLong(date.substring(0,4));
 	    month  = Long.parseLong(date.substring(4,6));
 	    day    = Long.parseLong(date.substring(6));
+            if (time.length() < 7) {
+              StringBuffer tmp = new StringBuffer(time);
+              for (int i = 0; i < (7 - time.length()); ++i)
+                tmp = tmp.append('0');
+              time = tmp.toString();
+            }
 	    fractionL = Long.parseLong(time);
 	    
 	    fraction = ((double) fractionL) / 10000000.0;
@@ -213,6 +388,37 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 	}
     }
 
+    /**
+     * 
+     * This function parses an array of date/time strings and returns an EPOCH
+     * value array.  The format must be exactly as shown below.  Note that if 
+     * there are less than 7 digits after the decimal point, zeros (0's)
+     * are assumed for the missing digits.
+     *
+     * <PRE>
+     * Format:    yyyymmdd.ttttttt
+     * Examples:  19950508.0000000
+     *            19671231.58      (== 19671213.5800000)
+     * </PRE>
+     *
+     * The expected format is the same as that produced by encode1.
+     *
+     * @param inStrings the epoch array in string representation
+     * @return the value array of the epoch represented by inString
+     *
+     * @exception  CDFException if a bad epoch value is passed in inString
+     */
+    public static double[] parse1 (String[] inStrings)
+        throws CDFException
+    {
+        if (inStrings == null) return null;
+        int len = inStrings.length;
+        double[] epochs = new double[len];
+        for (int i = 0; i < len; ++i)
+          epochs[i] = parse1 (inStrings[i]);
+        return epochs;
+    }
+
     /**
      * 
      * This function parses an input date/time string and returns an EPOCH
@@ -235,6 +441,7 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 	throws CDFException
     {
 	long year = 0, month = 0, day = 0, hour = 0, minute = 0, second = 0;
+        inString = inString.trim();
 	try {
 	    year   = Long.parseLong(inString.substring(0,4));
 	    month  = Long.parseLong(inString.substring(4,6));
@@ -248,6 +455,35 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 	}
     }
     
+    /**
+     * 
+     * This function parses an array of date/time strings and returns an EPOCH
+     * value array.  The format must be exactly as shown below.
+     *
+     * <PRE>
+     * Format:   yyyymmddhhmmss
+     * Examples: 19950508000000
+     *           19671231235959
+     * </PRE>
+     *
+     * The expected format is the same as that produced by encode2.
+     *
+     * @param inStrings the epoch array in string representation
+     * @return the value array of the epoch represented by inString
+     *
+     * @exception CDFException if a bad epoch value is passed in inString
+     */
+    public static double[] parse2 (String[] inStrings)
+        throws CDFException
+    {
+        if (inStrings == null) return null;
+        int len = inStrings.length;
+        double[] epochs = new double[len];
+        for (int i = 0; i < len; ++i)
+          epochs[i] = parse2(inStrings[i]);
+        return epochs;
+    }
+
     /**
      * 
      * This function parses an input date/time string and returns an EPOCH
@@ -270,13 +506,14 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 	throws CDFException
     {
 	long year = 0, month = 0, day = 0, hour = 0, 
-	    minute = 0, second = 0, msec = 0;
+	     minute = 0, second = 0, msec = 0;
 
 	String secondStr;
-
+        inString = inString.trim();
 	try {
 	    // Breakup the date and time fields
-	    StringTokenizer st = new StringTokenizer(inString, "T");
+            String tt0 = new String(new char[] {inString.charAt(10)});
+            StringTokenizer st = new StringTokenizer(inString, tt0);
 	    String date = st.nextToken();
 	    String tt = st.nextToken();
 	    String time = tt.substring(0,tt.length() - 1);
@@ -294,14 +531,43 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 	    secondStr = st.nextToken();
 	    st = new StringTokenizer(secondStr, ".");
 	    second = Long.parseLong(st.nextToken());
-	    msec   = Long.parseLong(st.nextToken());
-
+	    if (st.hasMoreTokens())
+	      msec   = Long.parseLong(st.nextToken());
 	    return compute(year, month, day, hour, minute, second, msec);
 	} catch (java.lang.Exception e) {
 	    throw new CDFException(ILLEGAL_EPOCH_FIELD);
 	}
     }
 
+    /**
+     * 
+     * This function parses an array of date/time strings and returns an EPOCH
+     * value array.  The format must be exactly as shown below.
+     *
+     * <PRE>
+     * Format:    yyyy-mm-ddThh:mm:ss.cccZ
+     * Examples:  1990-04-01T03:05:02.000Z
+     *            1993-10-10T23:45:49.999Z
+     * </PRE>
+     *
+     * The expected format is the same as that produced by encode3.
+     *
+     * @param inStrings the epoch array in string representation
+     * @return the value array of the epoch represented by inString
+     *
+     * @exception CDFException if a bad epoch value is passed in inString
+     */
+    public static double[] parse3 (String[] inStrings)
+        throws CDFException
+    {
+        if (inStrings == null) return null;
+        int len = inStrings.length;
+        double[] epochs = new double[len];
+        for (int i = 0; i < len; ++i)
+          epochs[i] = parse3 (inStrings[i]);
+        return epochs;
+    }
+
     /**
      * 
      * This function parses an input date/time string and returns an EPOCH
@@ -324,16 +590,17 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 	throws CDFException
     {
 	long year = 0, month = 0, day = 0, hour = 0, 
-	    minute = 0, second = 0, msec = 0;
+	     minute = 0, second = 0, msec = 0;
 
 	String secondStr;
-
+        inString = inString.trim();
 	try {
 	    // Breakup the date and time fields
-	    StringTokenizer st = new StringTokenizer(inString, "T");
+	    String tt0 = new String(new char[] {inString.charAt(10)});
+	    StringTokenizer st = new StringTokenizer(inString, tt0);
 	    String date = st.nextToken();
-	    String tt = st.nextToken();
-	    String time = tt.substring(0,tt.length() - 1);
+            String time = st.nextToken();
+//	    String time = tt.substring(0,tt.length());
 
 	    // Breakup the date portion
 	    st = new StringTokenizer(date, "-");
@@ -348,7 +615,8 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 	    secondStr = st.nextToken();
 	    st = new StringTokenizer(secondStr, ".");
 	    second = Long.parseLong(st.nextToken());
-	    msec   = Long.parseLong(st.nextToken());
+	    if (st.hasMoreTokens())
+	      msec   = Long.parseLong(st.nextToken());
 
 	    return compute(year, month, day, hour, minute, second, msec);
 	} catch (java.lang.Exception e) {
@@ -356,6 +624,261 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 	}
     }
 
+    /**
+     * 
+     * This function parses an array of date/time strings and returns an EPOCH
+     * value array.  The format must be an ISO8601 and is exactly as shown below.
+     *
+     * <PRE>
+     * Format:    yyyy-mm-ddThh:mm:ss.ccc
+     * Examples:  1990-04-01T03:05:02.000
+     *            1993-10-10T23:45:49.999
+     * </PRE>
+     *
+     * The expected format is the same as that produced by encode3.
+     *
+     * @param inStrings the epoch array in string representation
+     * @return the value array of the epoch represented by inString
+     *
+     * @exception CDFException if a bad epoch value is passed in inString
+     */
+    public static double[] parse4 (String[] inStrings)
+        throws CDFException
+    {
+        if (inStrings == null) return null;
+        int len = inStrings.length;
+        double[] epochs = new double[len];
+        for (int i = 0; i < len; ++i)
+          epochs[i] = parse4 (inStrings[i]);
+        return epochs;
+    }
+
+    /**
+     * 
+     * Converts an EPOCH value into a readable date/time string.
+     * Depending on the encoding style, it can present the string to
+     * one of the following forms: 
+     *
+     * <PRE>
+     * For style 0: 
+     * Format:    dd-mmm-yyyy hh:mm:ss.ccc
+     * Examples:  01-Apr-1990 03:05:02.000
+     *            10-Oct-1993 23:45:49.999
+     *
+     * For style 1: 
+     * Format:    yyyymmdd.ttttttt
+     * Examples:  19950508.0000000
+     *            19671231.58      (== 19671213.5800000)
+     *
+     * For style 2: 
+     * Format:    yyyymmddhhmmss
+     * Examples:  19900401030502
+     *            19931010234549
+     *
+     * For style 3: 
+     * Format:    yyyymmddThh:mm:ss.cccZ
+     * Examples:  19900401T03:05:02.000Z
+     *            19931010T23:45:49.999Z
+     *
+     * For style 4: 
+     * Format:    yyyymmddThh:mm:ss.ccc
+     * Examples:  19900401T03:05:02.000
+     *            19931010T23:45:49.999
+     * The default style if the style is invalid.
+     *
+     * </PRE>
+     *
+     * @param epoch the epoch value
+     * @param style the encoding style
+     *
+     * @return A string representation of the epoch
+     */
+    public static String toEncode (double epoch, int style)
+    {
+        if (style < 0 || style > 4) style = 4;
+        if (style == 0) return encode(epoch);
+        else if (style == 1) return encode1(epoch);
+        else if (style == 2) return encode2(epoch);
+        else if (style == 3) return encode3(epoch);
+        else return encode4(epoch);
+    }
+
+    /**
+     * 
+     * Converts an array of EPOCH values into readable date/time strings.
+     * Depending on the encoding style, it can present the string to
+     * one of the following forms: 
+     *
+     * <PRE>
+     * For style 0: 
+     * Format:    dd-mmm-yyyy hh:mm:ss.ccc
+     * Examples:  01-Apr-1990 03:05:02.000
+     *            10-Oct-1993 23:45:49.999
+     *
+     * For style 1: 
+     * Format:    yyyymmdd.ttttttt
+     * Examples:  19950508.0000000
+     *            19671231.58      (== 19671213.5800000)
+     *
+     * For style 2: 
+     * Format:    yyyymmddhhmmss
+     * Examples:  19900401030502
+     *            19931010234549
+     *
+     * For style 3: 
+     * Format:    yyyymmddThh:mm:ss.cccZ
+     * Examples:  19900401T03:05:02.000Z
+     *            19931010T23:45:49.999Z
+     *
+     * For style 4: 
+     * Format:    yyyymmddThh:mm:ss.ccc
+     * Examples:  19900401T03:05:02.000
+     *            19931010T23:45:49.999
+     *
+     * </PRE>
+     *
+     * @param epochs the epoch values
+     * @param style the encoding style
+     *
+     * @return An array of strings representation of the epochs
+     *         If the style is invalid, it will return the string
+     *         "99991231T23:59:59.999"
+     */
+    public static String[] toEncode (double[] epochs, int style)
+    {
+        if (epochs == null) return null;
+        int len = epochs.length;
+        String[] strings = new String[len];
+        for (int i = 0; i < len; ++i)
+          strings[i] = toEncode (epochs[i], style);
+        return strings;
+    }
+
+    /**
+     * 
+     * Converts an EPOCH value into a readable date/time string.
+     * It present the string to the following ISO 8601 form: 
+     *
+     * <PRE>
+     *
+     * Format:    yyyymmddThh:mm:ss.ccc
+     * Examples:  19900401T03:05:02.000
+     *            19931010T23:45:49.999
+     *
+     * </PRE>
+     *
+     * @param epoch the epoch value
+     *
+     * @return A string representation of the epoch
+     */
+    public static String toEncode (double epoch)
+    {
+        return toEncode(epoch, 4);
+    }
+
+    /**
+     * 
+     * Converts an array of EPOCH values into readable date/time strings.
+     * The strings are of the following ISO 8601 form: 
+     *
+     * <PRE>
+     *
+     * Format:    yyyymmddThh:mm:ss.ccc
+     * Examples:  19900401T03:05:02.000
+     *            19931010T23:45:49.999
+     *
+     * </PRE>
+     *
+     * @param epochs the epoch values
+     *
+     * @return A array of strings representation of the epochs
+     */
+    public static String[] toEncode (double[] epochs)
+    {
+        return toEncode(epochs, 4);
+    }
+
+    /**
+     * 
+     * Converts an EPOCH value to a unix time of a double value (seconds from 
+     * 1970-01-01 00:00:00 UTC). Resolution to milliseconds will be made in 
+     * unix time at its fraction part.
+     *
+     * @param epoch the epoch value
+     *
+     * @return A unix time value
+     */
+    public static double EpochtoUnixTime (double epoch)
+    {
+        try {
+	  return (epoch - BeginUnixTimeEPOCH)/1000.0;
+        } catch (Exception ex) {
+          return 0.0;
+        }
+    }
+
+    /**
+     * 
+     * Converts an array of EPOCH values to an array of unix time of a double
+     * value (seconds from 1970-01-01 00:00:00 UTC). Resolution to milliseconds 
+     * will be made in unix time at its fraction part.
+     *
+     * @param epoch the epoch values
+     *
+     * @return Array of unix time values
+     */
+    public static double[] EpochtoUnixTime (double[] epoch)
+    {
+        double[] unixTimes = new double[epoch.length];
+        for (int i = 0; i < epoch.length; ++i)
+          unixTimes[i] = (epoch[i] - BeginUnixTimeEPOCH)/1000.0;
+	return unixTimes;
+    }
+
+    /**
+     * 
+     * Converts a Unix time value (seconds from 1970-01-01 00:00:00 UTC) to
+     * an EPOCH time. The Unix time can have a time resoultion of milliseconds
+     * (0-999) or microseconds (0-999999) at its fraction part. Only resolution
+     * to milliseconds in unix time will be used.
+     *
+     * @param unixTime the unix time value
+     *
+     * @return An epoch value
+     */
+    public static double UnixTimetoEpoch (double unixTime)
+    {
+        try {
+          long seconds, milsecs; double subsecs;
+          seconds = (long) unixTime;
+          subsecs = (unixTime - seconds) * 1000.0;
+          milsecs = (long) subsecs;
+          if ((subsecs - milsecs) > 0.5) milsecs += 1;
+	  return (seconds*1000.0 + milsecs + BeginUnixTimeEPOCH);
+        } catch (Exception ex) {
+          return 0.0;
+        }
+    }
+
+    /**
+     * 
+     * Converts an array of Unix time values (seconds from 1970-01-01
+     * 00:00:00 UTC) to an array of EPOCH times. The Unix time can have a time
+     * resoultion of milliseconds (0-999) or microseconds (0-999999) at its 
+     * fraction part. Only resolution to milliseconds in unix time will be used.
+     *
+     * @param unixTimes the unix time values
+     *
+     * @return An array of epoch values
+     */
+    public static double[] UnixTimetoEpoch (double[] unixTimes)
+    {
+        double[] epoch = new double[unixTimes.length];
+        for (int i = 0; i < unixTimes.length; ++i)
+          epoch[i] = UnixTimetoEpoch (unixTimes[i]);
+        return epoch;
+    }
+
     /**
      * 
      * Converts an EPOCH value into a readable date/time string.
@@ -372,9 +895,9 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
      *
      * @return A string representation of the epoch
      */
-    public static String encode(double epoch)
+    public static String encode (double epoch)
     {
-        if (epoch == -1.0E31) {
+        if (epoch == -1.0E31 || epoch == ILLEGAL_EPOCH_VALUE) {
           return "31-Dec-9999 23:59:59.999";
         }
 
@@ -384,7 +907,7 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 
     /**
      * 
-     * Converts an array of EPOCH values into an array of  readable date/time
+     * Converts an array of EPOCH values into an array of readable date/time
      * strings.
      *
      * <PRE>
@@ -400,7 +923,7 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
      * @return Strings representing the epochs or null if null or an empty
      *         input array is detected
      */
-    public static String[] encode(double[] epochs)
+    public static String[] encode (double[] epochs)
     {
 	if (epochs == null) return null;
 	int len = epochs.length;
@@ -429,13 +952,39 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
      */
     public static String encode1 (double epoch)
     {
-        if (epoch == -1.0E31) {
+        if (epoch == -1.0E31 || epoch == ILLEGAL_EPOCH_VALUE) {
           return "99991231.9999999";
         }
         
 	return encodex (epoch, "<year><mm.02><dom.02>.<fod.7>");
     }
 
+    /**
+     * 
+     * Converts an array EPOCH values into readable date/time strings.
+     *
+     * <PRE>
+     * Format:    yyyymmdd.ttttttt
+     * Examples:  19900401.3658893
+     *            19611231.0000000
+     * </PRE>
+     *
+     * This format is the same as that expected by parse1.
+     *
+     * @param epochs the epoch values
+     *
+     * @return Array of strings representation of the epochs
+     */
+    public static String[] encode1 (double[] epochs)
+    {
+        if (epochs == null) return null;
+        int len = epochs.length;
+        String[] strings = new String[len];
+        for (int i = 0; i < len; ++i)
+          strings[i] = encode1 (epochs[i]);
+        return strings;
+    }
+
     /**
      * 
      * Converts an EPOCH value into a readable date/time string.
@@ -454,13 +1003,39 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
      */
     public static String encode2 (double epoch)
     {
-        if (epoch == -1.0E31) {
+        if (epoch == -1.0E31 || epoch == ILLEGAL_EPOCH_VALUE) {
           return "99991231235959";
         }
         
 	return encodex (epoch, "<year><mm.02><dom.02><hour><min><sec>");
     }
 
+    /**
+     * 
+     * Converts an array of EPOCH values into readable date/time strings.
+     *
+     * <PRE>
+     * Format:     yyyymmddhhmmss
+     * Examples:   19900401235959
+     *             19611231000000
+     * </PRE>
+     *
+     * This format is the same as that expected by parse2.
+     *
+     * @param epochs the epoch values
+     *
+     * @return A array of strings representation of the epochs
+     */
+    public static String[] encode2 (double[] epochs)
+    {
+        if (epochs == null) return null;
+        int len = epochs.length;
+        String[] strings = new String[len];
+        for (int i = 0; i < len; ++i)
+          strings[i] = encode2 (epochs[i]);
+        return strings;
+    }
+
     /**
      * 
      * Converts an EPOCH value into a readable date/time string.
@@ -479,7 +1054,7 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
      */
     public static String encode3 (double epoch)
     {
-        if (epoch == -1.0E31) {
+        if (epoch == -1.0E31 || epoch == ILLEGAL_EPOCH_VALUE) {
           return "9999-12-31T23:59:59.999Z";
         }
         
@@ -488,6 +1063,32 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 		     "<year>-<mm.02>-<dom.02>T<hour>:<min>:<sec>.<fos>Z");
     }
 
+    /**
+     * 
+     * Converts an array of EPOCH values into readable date/time strings.
+     *
+     * <PRE>
+     * Format:    yyyy-mm-ddThh:mm:ss.cccZ
+     * Examples:  1990-04-01T03:05:02.000Z
+     *            1993-10-10T23:45:49.999Z
+     * </PRE>
+     *
+     * This format is the same as that expected by parse3.
+     *
+     * @param epochs the epoch values
+     *
+     * @return A array of strings representation of the epochs
+     */
+    public static String[] encode3 (double[] epochs)
+    {
+        if (epochs == null) return null;
+        int len = epochs.length;
+        String[] strings = new String[len];
+        for (int i = 0; i < len; ++i)
+          strings[i] = encode3 (epochs[i]);
+        return strings;
+    }
+
     /**
      * 
      * Converts an EPOCH value into a readable date/time, ISO8601 string.
@@ -506,7 +1107,7 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
      */
     public static String encode4 (double epoch)
     {
-        if (epoch == -1.0E31) {
+        if (epoch == -1.0E31 || epoch == ILLEGAL_EPOCH_VALUE) {
           return "9999-12-31T23:59:59.999";
         }
         
@@ -515,6 +1116,33 @@ public class Epoch implements gsfc.nssdc.cdf.CDFConstants {
 		     "<year>-<mm.02>-<dom.02>T<hour>:<min>:<sec>.<fos>");
     }
 
+    /**
+     * 
+     * Converts an array of EPOCH values into readable date/time, ISO8601
+     * strings.
+     *
+     * <PRE>
+     * Format:    yyyy-mm-ddThh:mm:ss.ccc
+     * Examples:  1990-04-01T03:05:02.000
+     *            1993-10-10T23:45:49.999
+     * </PRE>
+     *
+     * This format is the same as that expected by parse3.
+     *
+     * @param epochs the epoch values
+     *
+     * @return A array of strings representation of the epochs
+     */
+    public static String[] encode4 (double[] epochs)
+    {
+        if (epochs == null) return null;
+        int len = epochs.length;
+        String[] strings = new String[len];
+        for (int i = 0; i < len; ++i)
+          strings[i] = encode4 (epochs[i]);
+        return strings;
+    }
+
     /**
      * 
      * Converts an EPOCH value into a readable date/time string using the
