@@ -1,4 +1,4 @@
-// $Id: Epoch16.java,v 1.1.1.1 2017/01/04 20:58:09 liu Exp $
+// $Id: Epoch16.java,v 1.17 2019/04/11 14:04:26 mhliu Exp $
 /*
  * Copyright 1996-2014 United States Government as represented by the
  * Administrator of the National Aeronautics and Space Administration.
@@ -28,7 +28,7 @@ import java.text.*;
  * double ep = Epoch16.compute(1990, 8, 5, 5, 0, 0, 0, 0, 0, 0, epoch16);
  * //Get the year, month, day, hour, minutes, seconds, milliseconds,
  * //    microseconds, nanaseconds and picoseconds for epoch16
- * long times[] = Epoch16.breakdown(epoch16);
+ * long[] times = Epoch16.breakdown(epoch16);
  * for (int i=0;i&lt;times.length;i++)
  *     System.out.print(times[i]+" ");
  * System.out.println();
@@ -65,6 +65,88 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
 	"Dec"
     };
 
+    /**
+     * 
+     * This function parses a date/time string and returns an
+     * EPOCH16 value.  The string format must be exactly as one of the
+     * followings (which are made from the corresponding encoding methods).
+     *
+     * <PRE>
+     * Style 0 - 
+     * Format:   dd-mmm-yyyy hh:mm:ss.mmm.uuu.nnn.ppp
+     * Examples:  1-Apr-1990 03:05:02.000.000.000.000
+     *           10-Oct-1993 23:45:49.999.999.999.999
+     *
+     * Style 1 - 
+     * Format:   yyyymmdd.ttttttttttttttt
+     * Examples: 19900401.365889312345678
+     *           19931010.000000000000000
+     *
+     * Style 2 - 
+     * Format:   yyyymmddhhmmss
+     * Examples: 19900401030502
+     *           19931010234549
+     *
+     * Style 3 - 
+     * Format:   yyyy-mm-ddThh:mm:ss.mmm.uuu.nnn.pppZ
+     * Examples: 1990-04-01T03:05:02.000.000.000.000Z
+     *           1993-10-10T23:45:49.999.999.999.999Z
+     *
+     * Style 4 - 
+     * Format:   yyyy-mm-ddThh:mm:ss.mmmuuunnnppp
+     * Examples: 1990-04-01T03:05:02.000000000000
+     *           1993-10-10T23:45:49.999999999999
+     *
+     * </PRE>
+     *
+     * @param inString the epoch in string representation
+     *
+     * @return the value of the epoch represented by inString or null if null
+     *         or an empty string is detected
+     *
+     * @exception CDFException if a bad epoch value is passed in inStrings
+     */
+    public static double[] toParse (String inString)
+        throws CDFException
+    {
+        if (inString == null) return null;
+        inString = inString.trim();
+        int len = inString.length();
+        char aC;
+        if (len == EPOCH_STRING_LEN_EXTEND) { // style 0 or style 3
+          if (inString.charAt(11) == ' ') return (double[]) parse(inString);
+          aC = inString.charAt(10);
+          if (aC == 'T' || aC == 't' || aC == ' ' || aC == '/') 
+            return (double[]) parse3(inString);
+          else return null;
+        } else if (len == EPOCH4_STRING_LEN_EXTEND) {
+          aC = inString.charAt(10);
+          if (aC == 'T' || aC == 't' || aC == ' ' || aC == '/')
+            return (double[]) parse4(inString);
+          else return new double[] {-1.0E31, -1.0E31};
+        } else if (len <= EPOCH1_STRING_LEN_EXTEND &&
+                   inString.charAt(8) == '.' &&
+                   inString.matches("\\d+.\\d+")) {
+          return (double[]) parse1(inString);
+        } else if (len == EPOCH2_STRING_LEN_EXTEND && 
+                   inString.matches("\\d+")) {
+          return (double[]) parse2(inString);
+        } else if (inString.charAt(11) == ' ') { // style 0, sort of
+          return (double[]) parse(inString);
+        } else if (inString.charAt(10) == 'T' || inString.charAt(10) == 't' ||
+                   inString.charAt(10) == ' ' || inString.charAt(10) == '/') {
+             if (inString.endsWith("Z") || inString.endsWith("z"))
+               return (double[]) parse3(inString);
+             else {
+               if (len == (EPOCH3_STRING_LEN_EXTEND - 1)) 
+                 return (double[]) parse3(inString);
+               else
+                 return (double[]) parse4(inString);
+             }
+        } else
+          return new double[] {-1.0E31, -1.0E31};
+    }
+
     /**
      * 
      * This function parses an input date/time string and returns an EPOCH16
@@ -85,7 +167,7 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
      *
      * @exception CDFException if a bad epoch value is passed in inString
      */
-    public static Object parse(String inString)
+    public static Object parse (String inString)
 	throws CDFException
     {
 	long 
@@ -101,8 +183,10 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
 	    psec   = 0;
 	double mmm;
 	double[]  epoch16 = new double[2];
+        inString = inString.trim();
 	String monthStr, secondStr;
-
+        if (inString.endsWith("Z") || inString.endsWith("z"))
+          inString = inString.substring(0, inString.length()-1);
 	try {
 	    StringTokenizer st = new StringTokenizer(inString, " ");
 	    String date = st.nextToken();
@@ -169,6 +253,7 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
 	throws CDFException
     {
 	StringTokenizer st = new StringTokenizer(inString, ".");
+        inString = inString.trim();
         try {
 	  String date = st.nextToken();
 	  String time = st.nextToken();
@@ -232,6 +317,7 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
 	long year = 0, month = 0, day = 0, hour = 0, minute = 0, second = 0;
 	double mmm;
 	double[] epoch16 = new double[2];
+        inString = inString.trim();
 	try {
 	    year   = Long.parseLong(inString.substring(0,4));
 	    month  = Long.parseLong(inString.substring(4,6));
@@ -276,13 +362,18 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
 	String secondStr;
 	double[] epoch16 = new double[2];
 	double mmm;
-
+        inString = inString.trim();
 	try {
 	    // Breakup the date and time fields
-    	    StringTokenizer st = new StringTokenizer(inString, "T");
+	    String tt0 = new String(new char[] {inString.charAt(10)});
+    	    StringTokenizer st = new StringTokenizer(inString, tt0);
 	    String date = st.nextToken();
 	    String tt = st.nextToken();
-	    String time = tt.substring(0,tt.length() - 1);
+	    String time;
+            if (inString.endsWith("Z") || inString.endsWith("z"))
+              time = tt.substring(0,tt.length() - 1);
+            else
+              time = tt.substring(0,tt.length());
 	    String fracSec;
 
 	    // Breakup the date portion
@@ -302,7 +393,6 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
 	    if (st.hasMoreTokens()) usec = Long.parseLong(st.nextToken());
 	    if (st.hasMoreTokens()) nsec = Long.parseLong(st.nextToken());
 	    if (st.hasMoreTokens()) psec = Long.parseLong(st.nextToken());
-
 	    mmm =  compute(year, month, day, hour, minute, second, 
 			   msec, usec,  nsec, psec, epoch16);
 	    if (mmm == ILLEGAL_EPOCH_VALUE)
@@ -324,7 +414,7 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
      *                    1993-10-10T23:45:49.999999999999
      * </PRE>
      *
-     * The expected format is the same as that produced by encode3.
+     * The expected format is the same as that produced by encode4.
      *
      * @param inString the epoch in string representation
      * @return the value of the epoch represented by inString
@@ -340,10 +430,11 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
 	String secondStr;
 	double[] epoch16 = new double[2];
 	double mmm;
-
+        inString = inString.trim();
 	try {
 	    // Breakup the date and time fields
-    	    StringTokenizer st = new StringTokenizer(inString, "T");
+	    String tt0 = new String(new char[] {inString.charAt(10)});
+    	    StringTokenizer st = new StringTokenizer(inString, tt0);
 	    String date = st.nextToken();
 	    String time = st.nextToken();
 	    String fracSec;
@@ -361,11 +452,14 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
 	    secondStr = st.nextToken();
 	    st = new StringTokenizer(secondStr, ".");
 	    second = Long.parseLong(st.nextToken());
-            String subSecs = st.nextToken();
-	    msec = Long.parseLong(subSecs.substring(0,3));
-	    usec = Long.parseLong(subSecs.substring(3,6));
-	    nsec = Long.parseLong(subSecs.substring(6,9));
-	    psec = Long.parseLong(subSecs.substring(9,12));
+            if (st.hasMoreTokens()) {
+              String subSecs = st.nextToken();
+              int subLen = subSecs.length();
+	      msec = Long.parseLong(subSecs.substring(0,3));
+              if (subLen > 3) usec = Long.parseLong(subSecs.substring(3,6));
+	      if (subLen > 6) nsec = Long.parseLong(subSecs.substring(6,9));
+	      if (subLen > 9) psec = Long.parseLong(subSecs.substring(9,12));
+            }
 	    mmm =  compute(year, month, day, hour, minute, second, 
 			   msec, usec,  nsec, psec, epoch16);
 	    if (mmm == ILLEGAL_EPOCH_VALUE)
@@ -376,6 +470,129 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
 	}
     }
 
+    /**
+     * 
+     * Converts an EPOCH16 value into a readable date/time string.
+     * Depending on the encoding style, it can present the string to
+     * one of the following forms: 
+     *
+     * <PRE>
+     * For style 0: 
+     * Format:    dd-mmm-yyyy hh:mm:ss.ccc.uuu.nnn.ppp
+     * Examples:  01-Apr-1990 03:05:02.000.111.222.333
+     *            10-Oct-1993 23:45:49.999.999.999.999
+     * 
+     * For style 1: 
+     * Format:    yyyymmdd.ttttttt
+     * Examples:  19950508.0000000
+     *            19671231.58      (== 19671213.5800000)
+     *  
+     * For style 2: 
+     * Format:    yyyymmddhhmmss
+     * Examples:  19900401030502 
+     *            19931010234549
+     *  
+     * For style 3: 
+     * Format:    yyyymmddThh:mm:ss.ccc.uuu.nnn.pppZ
+     * Examples:  19900401T03:05:02.000.111.222.333Z
+     *            19931010T23:45:49.999.999.999.999Z
+     *      
+     * For style 4: 
+     * Format:    yyyymmddThh:mm:ss.cccuuunnnppp
+     * Examples:  19900401T03:05:02.000111222333
+     *            19931010T23:45:49.999999999999
+     * The default style, if the style is invalid.
+     *      
+     * </PRE>
+     *      
+     * @param epoch the epoch value
+     * @param style the encoding style
+     *      
+     * @return A string representation of the epoch
+     */     
+    public static String toEncode(Object epoch, int style)
+    {       
+        if (style < 0 || style > 4) style = 4;
+        if (style == 0) return encode(epoch);
+        else if (style == 1) return encode1(epoch);
+        else if (style == 2) return encode2(epoch);
+        else if (style == 3) return encode3(epoch);
+        else return encode4(epoch);
+    }
+
+    /**
+     * 
+     * Converts an EPOCH value into a readable date/time string.
+     * It present the string to the following ISO 8601 form: 
+     *
+     * <PRE>
+     *
+     * Format:    yyyymmddThh:mm:ss.cccuuunnnppp
+     * Examples:  19900401T03:05:02.000111222333
+     *            19931010T23:45:49.999999999999
+     *
+     * </PRE>
+     *
+     * @param epoch the epoch value
+     *
+     * @return A string representation of the epoch
+     */
+    public static String toEncode(Object epoch)
+    {
+        return toEncode(epoch, 4);
+    }
+
+    /**
+     * 
+     * Converts an EPOCH16 value to a unix time of a double value (seconds from
+     * 1970-01-01 00:00:00 UTC). The unix time will have a time 
+     * resolution of microseconds at its fraction part.
+     *
+     * Note: As EPOCH16 have a higher time resolution than the unix time. The
+     * conversion might not precisely preserve the microseconds due to the 
+     * floating point presentation.
+     *
+     * @param epoch the epoch16 value (2-double)
+     *
+     * @return A unix time value
+     */
+    public static double Epoch16toUnixTime (Object epoch)
+    {
+        double seconds = ((double[])epoch)[0];
+        double subsecs = ((double[])epoch)[1];
+        double submics = subsecs / Math.pow(10.0, 6.0);
+        long   micsecs = (long) submics;
+        if ((submics - micsecs) > 0.5) micsecs += 1;
+	return (seconds-BeginUnixTimeEPOCH16+micsecs/Math.pow(10.0, 6.0));
+	
+    }
+
+    /**
+     * 
+     * Converts a unix time of a double value (seconds from 
+     * 1970-01-01 00:00:00 UTC) to an EPOCH16 value. The unix time can have a 
+     * time resolution in microseconds at its fraction part.
+     *
+     * Note: As EPOCH16 have a higher time resolution than the unix time. The
+     * conversion might not precisely preserve the microseconds due to the 
+     * floating point presentation.
+     * 
+     * @param unixTime the unix time value
+     *
+     * @return An EPOCH16 value (2-double)
+     */
+    public static double[] UnixTimetoEpoch16 (double unixTime)
+    {
+        double[] epoch = new double[2];
+        epoch[0] = BeginUnixTimeEPOCH16 + (long) unixTime;
+        double submics = (unixTime - (long) unixTime) * Math.pow(10.0, 6.0);
+        long micsecs = (long) submics;
+        if ((submics - micsecs) > 0.5) micsecs += 1;
+        epoch[1] = (double) micsecs * Math.pow(10.0, 6.0); 
+	return epoch;
+	
+    }
+
     /**
      * 
      * Converts an EPOCH16 value into a readable date/time string.
@@ -392,15 +609,14 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
      *
      * @return A string representation of the epoch
      */
-    public static String encode(Object epoch)
+    public static String encode (Object epoch)
     {
 	double[] epoch16 = new double[2];
 	String epString1 = new String(),
 	       epString2 = new String();
 	epoch16[0] = ((double[]) epoch)[0];
 	epoch16[1] = ((double[]) epoch)[1];
-
-	if ((epoch16[0] == -1.0E31) && (epoch16[1] == -1.0E31)) {
+	if (epoch16[0] == -1.0E31 && epoch16[1] == -1.0E31) {
           return "31-Dec-9999 23:59:59.999.999.999.999";
         }
 
@@ -433,7 +649,7 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
         double[] epoch16 = new double[2];
         epoch16[0] = ((double[]) epoch)[0];
         epoch16[1] = ((double[]) epoch)[1];
-        if ((epoch16[0] == -1.0E31) && (epoch16[1] == -1.0E31)) {
+        if (epoch16[0] == -1.0E31 && epoch16[1] == -1.0E31) {
           return "99991231.999999999999999";
         }
 
@@ -461,7 +677,7 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
         double[] epoch16 = new double[2];
         epoch16[0] = ((double[]) epoch)[0];
         epoch16[1] = ((double[]) epoch)[1];
-        if ((epoch16[0] == -1.0E31) && (epoch16[1] == -1.0E31)) {
+        if (epoch16[0] == -1.0E31 && epoch16[1] == -1.0E31) {
           return "99991231235959";
         }
 
@@ -489,7 +705,7 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
         double[] epoch16 = new double[2];
         epoch16[0] = ((double[]) epoch)[0];
         epoch16[1] = ((double[]) epoch)[1];
-        if ((epoch16[0] == -1.0E31) && (epoch16[1] == -1.0E31)) {
+        if (epoch16[0] == -1.0E31 && epoch16[1] == -1.0E31) {
           return "9999-12-31T23:59:59.999.999.999.999Z";
         }
 
@@ -519,7 +735,7 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
         double[] epoch16 = new double[2];
         epoch16[0] = ((double[]) epoch)[0];
         epoch16[1] = ((double[]) epoch)[1];
-        if ((epoch16[0] == -1.0E31) && (epoch16[1] == -1.0E31)) {
+        if (epoch16[0] == -1.0E31 && epoch16[1] == -1.0E31) {
           return "9999-12-31T23:59:59.999999999999";
         }
 
@@ -896,10 +1112,10 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
           if (day < 1L || day > 366L)
             return computeEpoch16(year,month,day,hour,minute,second,msec,usec,
                                   nsec, psec, epoch);
-	    daysSince0AD = (JulianDay(year,1L,1L) + (day-1)) - 1721060L;
 	} else {
           if (day < 1L || day > 31L)
-	    daysSince0AD = JulianDay(year,month,day) - 1721060L;
+            return computeEpoch16(year,month,day,hour,minute,second,msec,usec,
+                                  nsec, psec, epoch);
 	}
         if (month == 0L) {
           daysSince0AD = JulianDay(year,1L,1L) + (day-1) - 1721060L;
@@ -948,7 +1164,7 @@ public class Epoch16 implements gsfc.nssdc.cdf.CDFConstants {
 	epoch16[0] = ((double[]) epoch)[0];
 	epoch16[1] = ((double[]) epoch)[1];
 
-	if ((epoch16[0] == -1.0E31) && (epoch16[1] == -1.0E31)) {
+	if (epoch16[0] == -1.0E31 && epoch16[1] == -1.0E31) {
 	  components[0] = 9999;
 	  components[1] = 12;
 	  components[2] = 31;
